@@ -495,7 +495,8 @@ void HTMLTableElementImpl::parseAttribute(AttributeImpl *attr)
             padding = 1;
         if (m_render && m_render->isTable()) {
             static_cast<RenderTable *>(m_render)->setCellPadding(padding);
-            m_render->setNeedsLayout(true);
+	    if (!m_render->needsLayout())
+	        m_render->setNeedsLayout(true);
         }
         break;
     case ATTR_COLS:
@@ -599,7 +600,13 @@ void HTMLTablePartElementImpl::parseAttribute(AttributeImpl *attr)
     {
         DOMString v = attr->value();
         if ( strcasecmp( attr->value(), "middle" ) == 0 || strcasecmp( attr->value(), "center" ) == 0 )
-            addCSSProperty(CSS_PROP_TEXT_ALIGN, CSS_VAL__KONQ_CENTER);
+            addCSSProperty(CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_CENTER);
+        else if (strcasecmp(attr->value(), "absmiddle") == 0)
+            addCSSProperty(CSS_PROP_TEXT_ALIGN, CSS_VAL_CENTER);
+        else if (strcasecmp(attr->value(), "left") == 0)
+            addCSSProperty(CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_LEFT);
+        else if (strcasecmp(attr->value(), "right") == 0)
+            addCSSProperty(CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_RIGHT);
         else
             addCSSProperty(CSS_PROP_TEXT_ALIGN, v);
         break;
@@ -857,7 +864,7 @@ void HTMLTableCellElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     case ATTR_NOWRAP:
         if (attr->val() != 0)
-            addCSSProperty(CSS_PROP_WHITE_SPACE, CSS_VAL__KONQ_NOWRAP);
+            addCSSProperty(CSS_PROP_WHITE_SPACE, CSS_VAL__KHTML_NOWRAP);
         else
             removeCSSProperty(CSS_PROP_WHITE_SPACE);
         break;
