@@ -66,23 +66,13 @@ RenderBox::RenderBox(DOM::NodeImpl* node)
 
 void RenderBox::setStyle(RenderStyle *_style)
 {
-    // Make sure the root element retains its display:block type even across style
-    // changes.
-    if (isRoot() && _style->display() != NONE)
-        _style->setDisplay(BLOCK);
-    
     RenderObject::setStyle(_style);
 
     // The root always paints its background/border.
     if (isRoot())
         setShouldPaintBackgroundOrBorder(true);
-    
-    // ### move this into the parser. --> should work. Lars
-    // if only horizontal position was defined, vertical should be 50%
-    //if(!_style->backgroundXPosition().isVariable() && _style->backgroundYPosition().isVariable())
-    //style()->setBackgroundYPosition(Length(50, Percent));
 
-    setInline(style()->display()==INLINE);
+    setInline(_style->isDisplayInlineType());
     
     switch(_style->position())
     {
@@ -92,12 +82,14 @@ void RenderBox::setStyle(RenderStyle *_style)
         break;
     default:
         setPositioned(false);
-        if(_style->isFloating())
+
+        if (_style->isFloating())
             setFloating(true);
-        else if(_style->position() == RELATIVE)
+
+        if (_style->position() == RELATIVE)
             setRelPositioned(true);
     }
-    
+
     if (requiresLayer()) {
         if (!m_layer) {
             m_layer = new (renderArena()) RenderLayer(this);
@@ -109,17 +101,15 @@ void RenderBox::setStyle(RenderStyle *_style)
         m_layer = 0;
     }
 
-    if (m_layer) {
-        // Make sure our z-index values are only applied if we're positioned or
-        // relpositioned.
-        if (!isPositioned() && !isRelPositioned()) {
-            // Set the auto z-index flag.
-            if (isRoot())
-                style()->setZIndex(0);
-            else
-                style()->setHasAutoZIndex();
-        }
-    }
+    if (m_layer)
+        m_layer->styleChanged();
+    
+    // Set the text color if we're the body.
+    if (isBody())
+        element()->getDocument()->setTextColor(_style->color());
+    
+    if (style()->outlineWidth() > 0 && style()->outlineSize() > maximalOutlineSize(PaintActionOutline))
+        static_cast<RenderCanvas*>(document()->renderer())->setMaximalOutlineSize(style()->outlineSize());
 }
 
 RenderBox::~RenderBox()
@@ -127,14 +117,16 @@ RenderBox::~RenderBox()
     //kdDebug( 6040 ) << "Element destructor: this=" << nodeName().string() << endl;
 }
 
-void RenderBox::detach(RenderArena* renderArena)
+void RenderBox::detach()
 {
     RenderLayer* layer = m_layer;
+
+    RenderArena* arena = renderArena();
     
-    RenderContainer::detach(renderArena);
+    RenderContainer::detach();
     
     if (layer)
-        layer->detach(renderArena);
+        layer->detach(arena);
 }
 
 short RenderBox::contentWidth() const
@@ -154,6 +146,9 @@ int RenderBox::contentHeight() const
     int h = m_height - borderTop() - borderBottom();
     h -= paddingTop() + paddingBottom();
 
+    if (style()->scrollsOverflow() && m_layer)
+        h -= m_layer->horizontalScrollbarHeight();
+
     return h;
 }
 
@@ -193,12 +188,6 @@ void RenderBox::paint(QPainter *p, int _x, int _y, int _w, int _h,
     }
 }
 
-void RenderBox::setPixmap(const QPixmap &, const QRect&, CachedImage *image)
-{
-    if(image && image->pixmap_size() == image->valid_rect().size() && parent())
-        repaint();      //repaint bg when it finished loading
-}
-
 void RenderBox::paintRootBoxDecorations(QPainter *p,int, int _y,
                                         int, int _h, int _tx, int _ty)
 {
@@ -206,16 +195,40 @@ void RenderBox::paintRootBoxDecorations(QPainter *p,int, int _y,
     QColor c = style()->backgroundColor();
     CachedImage *bg = style()->backgroundImage();
 
-    if (!c.isValid() && !bg && firstChild()) {
-        if (!c.isValid())
-            c = firstChild()->style()->backgroundColor();
-        if (!bg)
-            bg = firstChild()->style()->backgroundImage();
+    bool canBeTransparent = true;
+    if (!c.isValid() && !bg) {
+        // Locate the <body> element using the DOM.  This is easier than trying
+        // to crawl around a render tree with potential :before/:after content and
+        // anonymous blocks created by inline <body> tags etc.  We can locate the <body>
+        // render object very easily via the DOM.
+        RenderObject* bodyObject = 0;
+        for (DOM::NodeImpl* elt = element()->firstChild(); elt; elt = elt->nextSibling()) {
+            if (elt->id() == ID_BODY) {
+                bodyObject = elt->renderer();
+                break;
+            }
+            else if (elt->id() == ID_FRAMESET) {
+                canBeTransparent = false; // Can't scroll a frameset document anyway.
+                break;
+            }
+        }
+
+        if (bodyObject) {
+            c = bodyObject->style()->backgroundColor();
+            bg = bodyObject->style()->backgroundImage();
+        }
     }
 
-    if( !c.isValid() && canvas()->view())
-        c = canvas()->view()->palette().active().color(QColorGroup::Base);
-
+    // Only fill with a base color (e.g., white) if we're the root document, since iframes/frames with
+    // no background in the child document should show the parent's background.
+    if ((!c.isValid() || qAlpha(c.rgb()) == 0) && canvas()->view()) {
+        DOM::NodeImpl* elt = element()->getDocument()->ownerElement();
+        if (canBeTransparent && elt && elt->id() != ID_FRAME) // Frames are never transparent.
+            canvas()->view()->useSlowRepaints(); // The parent must show behind the child.
+        else
+            c = canvas()->view()->palette().active().color(QColorGroup::Base);
+    }
+    
     int w = width();
     int h = height();
 
@@ -246,7 +259,7 @@ void RenderBox::paintRootBoxDecorations(QPainter *p,int, int _y,
 
     paintBackground(p, c, bg, my, _h, bx, by, bw, bh);
 
-    if(style()->hasBorder())
+    if (style()->hasBorder() && style()->display() != INLINE)
         paintBorder( p, _tx, _ty, w, h, style() );
 }
 
@@ -268,9 +281,15 @@ void RenderBox::paintBoxDecorations(QPainter *p,int _x, int _y,
     else
         mh = QMIN(_h,h);
 
-    paintBackground(p, style()->backgroundColor(), style()->backgroundImage(), my, mh, _tx, _ty, w, h);
-
-    if(style()->hasBorder())
+    // The <body> only paints its background if the root element has defined a background
+    // independent of the body.  Go through the DOM to get to the root element's render object,
+    // since the root could be inline and wrapped in an anonymous block.
+    if (!isBody()
+        || element()->getDocument()->documentElement()->renderer()->style()->backgroundColor().isValid()
+        || element()->getDocument()->documentElement()->renderer()->style()->backgroundImage())
+        paintBackground(p, style()->backgroundColor(), style()->backgroundImage(), my, mh, _tx, _ty, w, h);
+   
+    if (style()->hasBorder())
         paintBorder(p, _tx, _ty, w, h, style());
 }
 
@@ -284,8 +303,14 @@ void RenderBox::paintBackgroundExtended(QPainter *p, const QColor &c, CachedImag
                                         int _tx, int _ty, int w, int h,
                                         int bleft, int bright)
 {
-    if(c.isValid())
+    if (c.isValid() && qAlpha(c.rgb()) > 0) {
+        // If we have an alpha and we are painting the root element, go ahead and blend with our default
+        // background color (typically white).
+        if (qAlpha(c.rgb()) < 0xFF && isRoot())
+            p->fillRect(_tx, clipy, w, cliph, canvas()->view()->palette().active().color(QColorGroup::Base));
         p->fillRect(_tx, clipy, w, cliph, c);
+    }
+    
     // no progressive loading of the background image
     if(bg && bg->pixmap_size() == bg->valid_rect().size() && !bg->isTransparent() && !bg->isErrorImage()) {
         //kdDebug( 6040 ) << "painting bgimage at " << _tx << "/" << _ty << endl;
@@ -329,18 +354,17 @@ void RenderBox::paintBackgroundExtended(QPainter *p, const QColor &c, CachedImag
                         cw += xPosition;
                     }
                 }
+                cx += bleft;
             } else {
-                cw = w-vpab;
+                cw = w;
                 cx = _tx;
-                if(pixw == 0){
+                if (pixw == 0)
                     sx = 0;
-                }else{
+                else
                     sx =  pixw - ((sptr->backgroundXPosition().minWidth(pw-pixw)) % pixw );
-                }
+                sx -= bleft % pixw;
             }
 
-            cx += bleft;
-
             if( (bgr == NO_REPEAT || bgr == REPEAT_X) && h > pixh ) {
                 ch = pixh;
                 int yPosition = sptr->backgroundYPosition().minWidth(ph-pixh);
@@ -355,17 +379,18 @@ void RenderBox::paintBackgroundExtended(QPainter *p, const QColor &c, CachedImag
                         ch += yPosition;
                     }
                 }
+                
+                cy += borderTop();
             } else {
-                ch = h-hpab;
+                ch = h;
                 cy = _ty;
                 if(pixh == 0){
                     sy = 0;
                 }else{
                     sy = pixh - ((sptr->backgroundYPosition().minWidth(ph-pixh)) % pixh );
                 }
+                sy -= borderTop() % pixh;
             }
-
-            cy += borderTop();
         }
         else
         {
@@ -377,7 +402,7 @@ void RenderBox::paintBackgroundExtended(QPainter *p, const QColor &c, CachedImag
             int pixw = bg->pixmap_size().width();
             int pixh = bg->pixmap_size().height();
             EBackgroundRepeat bgr = sptr->backgroundRepeat();
-            if( (bgr == NO_REPEAT || bgr == REPEAT_Y) && w > pixw ) {
+            if( (bgr == NO_REPEAT || bgr == REPEAT_Y) && pw > pixw ) {
                 cw = pixw;
                 cx = vr.x() + sptr->backgroundXPosition().minWidth(pw-pixw);
             } else {
@@ -390,7 +415,7 @@ void RenderBox::paintBackgroundExtended(QPainter *p, const QColor &c, CachedImag
                 }
             }
 
-            if( (bgr == NO_REPEAT || bgr == REPEAT_X) && h > pixh ) {
+            if( (bgr == NO_REPEAT || bgr == REPEAT_X) && ph > pixh ) {
                 ch = pixh;
                 cy = vr.y() + sptr->backgroundYPosition().minWidth(ph-pixh);
             } else {
@@ -404,7 +429,7 @@ void RenderBox::paintBackgroundExtended(QPainter *p, const QColor &c, CachedImag
             }
 
             QRect fix(cx,cy,cw,ch);
-            QRect ele(_tx+bleft,_ty+borderTop(),w-vpab,h-hpab);
+            QRect ele(_tx,_ty,w,h);
             QRect b = fix.intersect(ele);
             sx+=b.x()-cx;
             sy+=b.y()-cy;
@@ -481,19 +506,12 @@ QRect RenderBox::getClipRect(int tx, int ty)
     return cr;
 }
 
-void RenderBox::close()
-{
-    setNeedsLayoutAndMinMaxRecalc();
-}
-
 short RenderBox::containingBlockWidth() const
 {
-    if (isRoot() && canvas()->view())
-        return canvas()->view()->visibleWidth();
-    
     RenderBlock* cb = containingBlock();
-    if ((style()->hidesOverflow() || ((style()->htmlHacks() || isTable()) && style()->flowAroundFloats()))
-            && style()->width().isVariable())
+    if (!cb)
+        return 0;
+    if (usesLineWidth())
         return cb->lineWidth(m_y);
     else
         return cb->contentWidth();
@@ -506,11 +524,15 @@ bool RenderBox::absolutePosition(int &xPos, int &yPos, bool f)
     RenderObject *o = container();
     if( o && o->absolutePosition(xPos, yPos, f))
     {
+        if (o->style()->hidesOverflow() && o->layer())
+            o->layer()->subtractScrollOffset(xPos, yPos); 
+            
         if(!isInline() || isReplaced())
             xPos += m_x, yPos += m_y;
 
         if(isRelPositioned())
             relativePositionOffset(xPos, yPos);
+
         return true;
     }
     else
@@ -551,52 +573,75 @@ void RenderBox::position(InlineBox* box, int from, int len, bool reverse)
     }
 }
 
-void RenderBox::repaint(bool immediate)
+QRect RenderBox::getAbsoluteRepaintRect()
 {
-    //kdDebug( 6040 ) << "repaint!" << endl;
-    if (isRoot()) {
-        RenderObject *cb = containingBlock();
-        if(cb != this)
-            cb->repaint(immediate);
-        return;
-    }
-    int ow = style() ? style()->outlineWidth() : 0;
-    repaintRectangle(-ow, -ow, overflowWidth(false)+ow*2, overflowHeight(false)+ow*2, immediate);
+    int ow = style() ? style()->outlineSize() : 0;
+    QRect r(-ow, -ow, overflowWidth(false)+ow*2, overflowHeight(false)+ow*2);
+    computeAbsoluteRepaintRect(r);
+    return r;
 }
 
-void RenderBox::repaintRectangle(int x, int y, int w, int h, bool immediate, bool f)
+void RenderBox::computeAbsoluteRepaintRect(QRect& r, bool f)
 {
-    x += m_x;
-    y += m_y;
-    
+    int x = r.x() + m_x;
+    int y = r.y() + m_y;
+     
     // Apply the relative position offset when invalidating a rectangle.  The layer
     // is translated, but the render box isn't, so we need to do this to get the
-    // right dirty rect.
-    if (isRelPositioned())
+    // right dirty rect.  Since this is called from RenderObject::setStyle, the relative position
+    // flag on the RenderObject has been cleared, so use the one on the style().
+#ifdef INCREMENTAL_REPAINTING
+    if (style()->position() == RELATIVE && m_layer)
+        m_layer->relativePositionOffset(x,y);
+#else
+    if (style()->position() == RELATIVE)
         relativePositionOffset(x,y);
+#endif
     
-    if (style()->position()==FIXED) f=true;
+    if (style()->position()==FIXED)
+        f = true;
 
     RenderObject* o = container();
     if (o) {
-        if (o->style()->hidesOverflow()) {
-            int ow = o->style() ? o->style()->outlineWidth() : 0;
-            QRect boxRect(-ow, -ow, o->width()+ow*2, o->height()+ow*2);
-            if (o->layer())
-                o->layer()->subtractScrollOffset(x,y); // For overflow:auto/scroll/hidden.
-            QRect repaintRect(x, y, w, h);
-            if (!repaintRect.intersects(boxRect))
+        // <body> may not have a layer, since it might be applying its overflow value to the
+        // scrollbars.
+        if (o->style()->hidesOverflow() && o->layer()) {
+            // o->height() is inaccurate if we're in the middle of a layout of |o|, so use the
+            // layer's size instead.  Even if the layer's size is wrong, the layer itself will repaint
+            // anyway if its size does change.
+            QRect boxRect(0, 0, o->layer()->width(), o->layer()->height());
+            o->layer()->subtractScrollOffset(x,y); // For overflow:auto/scroll/hidden.
+            QRect repaintRect(x, y, r.width(), r.height());
+            r = repaintRect.intersect(boxRect);
+            if (r.isEmpty())
                 return;
-            repaintRect = repaintRect.intersect(boxRect);
-            x = repaintRect.x();
-            y = repaintRect.y();
-            w = repaintRect.width();
-            h = repaintRect.height();
         }
-        o->repaintRectangle(x, y, w, h, immediate, f);
+        else {
+            r.setX(x);
+            r.setY(y);
+        }
+        o->computeAbsoluteRepaintRect(r, f);
     }
 }
 
+#ifdef INCREMENTAL_REPAINTING
+void RenderBox::repaintDuringLayoutIfMoved(int oldX, int oldY)
+{
+    int newX = m_x;
+    int newY = m_y;
+    if (oldX != newX || oldY != newY) {
+        // The child moved.  Invalidate the object's old and new positions.  We have to do this
+        // since the object may not have gotten a layout.
+        m_x = oldX; m_y = oldY;
+        repaint();
+        repaintFloatingDescendants();
+        m_x = newX; m_y = newY;
+        repaint();
+        repaintFloatingDescendants();
+    }
+}
+#endif
+
 void RenderBox::relativePositionOffset(int &tx, int &ty)
 {
     if(!style()->left().isVariable())
@@ -628,8 +673,18 @@ void RenderBox::calcWidth()
     }
     else
     {
+        // The parent box is flexing us, so it has increased or decreased our width.  We just bail
+        // and leave our width unchanged in this case.
+        if (parent()->isFlexibleBox() && parent()->style()->boxOrient() == HORIZONTAL
+            && parent()->isFlexingChildren())
+            return;
+
+        bool inVerticalBox = parent()->isFlexibleBox() && parent()->style()->boxOrient() == VERTICAL;
+        bool stretching = parent()->style()->boxAlign() == BSTRETCH;
+        bool treatAsReplaced = isReplaced() && !isInlineBlockOrInlineTable() &&
+            (!inVerticalBox || !stretching);
         Length w;
-        if ( isReplaced () )
+        if (treatAsReplaced)
             w = Length( calcReplacedWidth(), Fixed );
         else
             w = style()->width();
@@ -637,24 +692,20 @@ void RenderBox::calcWidth()
         Length ml = style()->marginLeft();
         Length mr = style()->marginRight();
 
-        int cw;
         RenderBlock *cb = containingBlock();
-        if (style()->hidesOverflow() || style()->flowAroundFloats())
-            cw = cb->lineWidth( m_y );
-        else
-            cw = cb->contentWidth();
+        int cw = containingBlockWidth();
 
         if (cw<0) cw = 0;
 
         m_marginLeft = 0;
         m_marginRight = 0;
 
-        if (isInline())
+        if (isInline() && !isInlineBlockOrInlineTable())
         {
             // just calculate margins
             m_marginLeft = ml.minWidth(cw);
             m_marginRight = mr.minWidth(cw);
-            if (isReplaced())
+            if (treatAsReplaced)
             {
                 m_width = w.width(cw);
                 m_width += paddingLeft() + paddingRight() + borderLeft() + borderRight();
@@ -665,7 +716,7 @@ void RenderBox::calcWidth()
         }
         else {
             LengthType widthType, minWidthType, maxWidthType;
-            if (isReplaced()) {
+            if (treatAsReplaced) {
                 m_width = w.width(cw);
                 m_width += paddingLeft() + paddingRight() + borderLeft() + borderRight();
                 widthType = w.type;
@@ -697,9 +748,10 @@ void RenderBox::calcWidth()
             }
         }
 
-        if (cw && cw != m_width + m_marginLeft + m_marginRight && !isFloating() && !isInline())
+        if (cw && cw != m_width + m_marginLeft + m_marginRight && !isFloating() && !isInline() &&
+            !cb->isFlexibleBox())
         {
-            if (style()->direction()==LTR)
+            if (cb->style()->direction()==LTR)
                 m_marginRight = cw - m_width - m_marginLeft;
             else
                 m_marginLeft = cw - m_width - m_marginRight;
@@ -748,7 +800,7 @@ int RenderBox::calcWidthUsing(WidthType widthType, int cw, LengthType& lengthTyp
 
 void RenderBox::calcHorizontalMargins(const Length& ml, const Length& mr, int cw)
 {
-    if (isFloating())
+    if (isFloating() || isInline()) // Inline blocks/tables and floats don't have their margins increased.
     {
         m_marginLeft = ml.minWidth(cw);
         m_marginRight = mr.minWidth(cw);
@@ -756,19 +808,23 @@ void RenderBox::calcHorizontalMargins(const Length& ml, const Length& mr, int cw
     else
     {
         if ( (ml.type == Variable && mr.type == Variable) ||
-             (!(ml.type == Variable) &&
-                containingBlock()->style()->textAlign() == KONQ_CENTER) )
+             (ml.type != Variable && mr.type != Variable &&
+                containingBlock()->style()->textAlign() == KHTML_CENTER) )
         {
             m_marginLeft = (cw - m_width)/2;
             if (m_marginLeft<0) m_marginLeft=0;
             m_marginRight = cw - m_width - m_marginLeft;
         }
-        else if (mr.type == Variable)
+        else if (mr.type == Variable ||
+                 (ml.type != Variable && containingBlock()->style()->direction() == RTL &&
+                  containingBlock()->style()->textAlign() == KHTML_LEFT))
         {
             m_marginLeft = ml.width(cw);
             m_marginRight = cw - m_width - m_marginLeft;
         }
-        else if (ml.type == Variable)
+        else if (ml.type == Variable ||
+                 (mr.type != Variable && containingBlock()->style()->direction() == LTR &&
+                  containingBlock()->style()->textAlign() == KHTML_RIGHT))
         {
             m_marginRight = mr.width(cw);
             m_marginLeft = cw - m_width - m_marginRight;
@@ -797,7 +853,17 @@ void RenderBox::calcHeight()
     else
     {
         Length h;
-        if ( isReplaced() ) {
+        bool inHorizontalBox = parent()->isFlexibleBox() && parent()->style()->boxOrient() == HORIZONTAL;
+        bool stretching = parent()->style()->boxAlign() == BSTRETCH;
+        
+        // The parent box is flexing us, so it has increased or decreased our height.  We have to
+        // grab our cached flexible height.
+        if (parent()->isFlexibleBox() && parent()->style()->boxOrient() == VERTICAL
+            && parent()->isFlexingChildren() && style()->boxFlexedHeight() != -1)
+            h = Length(style()->boxFlexedHeight() - borderTop() - borderBottom() -
+                       paddingTop() - paddingBottom(), Fixed);
+        else if ( isReplaced() && !isInlineBlockOrInlineTable() &&
+                  (!inHorizontalBox || !stretching )) {
             h = Length( calcReplacedHeight(), Fixed );
         }
         else
@@ -809,6 +875,19 @@ void RenderBox::calcHeight()
         if (isTable())
             return;
 
+        // The parent box is flexing us, so it has increased or decreased our height.  We have to
+        // grab our cached flexible height.
+        if (parent()->isFlexibleBox() && parent()->style()->boxOrient() == VERTICAL
+            && parent()->isFlexingChildren() && style()->boxFlexedHeight() != -1)
+            h = Length(style()->boxFlexedHeight() - borderTop() - borderBottom() -
+                       paddingTop() - paddingBottom(), Fixed);
+        
+        // Block children of horizontal flexible boxes fill the height of the box.
+        if (h.isVariable() && parent()->isFlexibleBox() && parent()->style()->boxOrient() == HORIZONTAL
+            && parent()->isStretchingChildren())
+            h = Length(parent()->contentHeight() - marginTop() - marginBottom() -
+                       borderTop() - paddingTop() - borderBottom() - paddingBottom(), Fixed);
+
         if (!h.isVariable())
         {
             int fh=-1;
@@ -846,13 +925,41 @@ void RenderBox::calcHeight()
             }
         }
     }
+    
+    // Unfurling marquees override with the furled height.
+    if (style()->overflow() == OMARQUEE && m_layer && m_layer->marquee() && 
+        m_layer->marquee()->isUnfurlMarquee() && !m_layer->marquee()->isHorizontal()) {
+        m_layer->marquee()->setEnd(m_height);
+        m_height = kMin(m_height, m_layer->marquee()->unfurlPos());
+    }
 }
 
 short RenderBox::calcReplacedWidth() const
 {
-   Length w = style()->width();
+    int width = calcReplacedWidthUsing(Width);
+    int minW = calcReplacedWidthUsing(MinWidth);
+    int maxW = style()->maxWidth().value == UNDEFINED ? width : calcReplacedWidthUsing(MaxWidth);
 
-   switch( w.type ) {
+    if (width > maxW)
+        width = maxW;
+    
+    if (width < minW)
+        width = minW;
+
+    return width;
+}
+
+int RenderBox::calcReplacedWidthUsing(WidthType widthType) const
+{
+    Length w;
+    if (widthType == Width)
+        w = style()->width();
+    else if (widthType == MinWidth)
+        w = style()->minWidth();
+    else
+        w = style()->maxWidth();
+    
+    switch (w.type) {
     case Fixed:
         return w.value;
     case Percent:
@@ -871,12 +978,31 @@ short RenderBox::calcReplacedWidth() const
 
 int RenderBox::calcReplacedHeight() const
 {
-    const Length& h = style()->height();
+    int height = calcReplacedHeightUsing(Height);
+    int minH = calcReplacedHeightUsing(MinHeight);
+    int maxH = style()->maxHeight().value == UNDEFINED ? height : calcReplacedHeightUsing(MaxHeight);
+
+    if (height > maxH)
+        height = maxH;
+
+    if (height < minH)
+        height = minH;
+
+    return height;
+}
+
+int RenderBox::calcReplacedHeightUsing(HeightType heightType) const
+{
+    Length h;
+    if (heightType == Height)
+        h = style()->height();
+    else if (heightType == MinHeight)
+        h = style()->minHeight();
+    else
+        h = style()->maxHeight();
     switch( h.type ) {
-    case Percent: {
-        int ah = availableHeight();
-        return ah;
-    }
+    case Percent:
+        return availableHeightUsing(h);
     case Fixed:
         return h.value;
     default:
@@ -886,8 +1012,11 @@ int RenderBox::calcReplacedHeight() const
 
 int RenderBox::availableHeight() const
 {
-    Length h = style()->height();
+    return availableHeightUsing(style()->height());
+}
 
+int RenderBox::availableHeightUsing(const Length& h) const
+{
     if (h.isFixed())
         return h.value;
 
@@ -946,8 +1075,9 @@ void RenderBox::calcAbsoluteHorizontal()
     int pab = borderLeft()+ borderRight()+ paddingLeft()+ paddingRight();
 
     l=r=ml=mr=w=AUTO;
-
-    RenderBlock* cb = containingBlock();
+ 
+    // We don't use containingBlock(), since we may be positioned by an enclosing relpositioned inline.
+    RenderObject* cb = container();
     cw = containingBlockWidth() + cb->paddingLeft() + cb->paddingRight();
 
     if(!style()->left().isVariable())
@@ -983,7 +1113,6 @@ void RenderBox::calcAbsoluteHorizontal()
             || style()->right().isStatic())
     {
         RenderObject* po = parent();
-        RenderBlock* cb = containingBlock();
         static_distance = m_staticX - cb->borderLeft(); // Should already have been set through layout of the parent().
         while (po && po!=containingBlock()) {
             static_distance+=po->xPos();
@@ -1032,7 +1161,9 @@ void RenderBox::calcAbsoluteHorizontal()
         //1. solve left & width.
         if (l==AUTO && w==AUTO && r!=AUTO)
         {
-            w = QMIN(m_maxWidth-pab, cw - ( r + ml + mr + pab));
+            // From section 10.3.7 of the CSS2.1 specification.
+            // "The shrink-to-fit width is: min(max(preferred minimum width, available width), preferred width)."
+            w = QMIN(QMAX(m_minWidth-pab, cw - ( r + ml + mr + pab)), m_maxWidth-pab);
             l = cw - ( r + w + ml + mr + pab);
         }
         else
@@ -1057,7 +1188,9 @@ void RenderBox::calcAbsoluteHorizontal()
         //3. solve width & right.
         if (l!=AUTO && w==AUTO && r==AUTO)
         {
-            w = QMIN(m_maxWidth-pab, cw - ( l + ml + mr + pab));
+            // From section 10.3.7 of the CSS2.1 specification.
+            // "The shrink-to-fit width is: min(max(preferred minimum width, available width), preferred width)."
+            w = QMIN(QMAX(m_minWidth-pab, cw - ( l + ml + mr + pab)), m_maxWidth-pab);
             r = cw - ( l + w + ml + mr + pab);
         }
         else
@@ -1101,20 +1234,20 @@ void RenderBox::calcAbsoluteVertical()
 
     int pab = borderTop()+borderBottom()+paddingTop()+paddingBottom();
 
-    RenderObject* cb = containingBlock();
+    // We don't use containingBlock(), since we may be positioned by an enclosing relpositioned inline.
+    RenderObject* cb = container();
     Length hl = cb->style()->height();
     if (hl.isFixed())
-        ch = hl.value + cb->paddingTop()
-             + cb->paddingBottom() + cb->borderTop() + cb->borderBottom();
+        ch = hl.value + cb->paddingTop() + cb->paddingBottom();
     else if (cb->isRoot())
         ch = cb->availableHeight();
     else
-        ch = cb->height();
+        ch = cb->height() - cb->borderTop() - cb->borderBottom();
 
     if(!style()->top().isVariable())
-        t = style()->top().width(ch) + cb->borderTop();
+        t = style()->top().width(ch);
     if(!style()->bottom().isVariable())
-        b = style()->bottom().width(ch) + cb->borderBottom();
+        b = style()->bottom().width(ch);
     if (isTable() && style()->height().isVariable())
         // Height is never unsolved for tables. "auto" means shrink to fit.  Use our
         // height instead.
@@ -1143,7 +1276,7 @@ void RenderBox::calcAbsoluteVertical()
         // used for 1) top=static-position
         //          2) top, bottom, height are all auto -> calc top -> 3.
         //          3) precalc for case 2 below
-        static_top = m_staticY; // Should already have been set through layout of the parent().
+        static_top = m_staticY - cb->borderTop(); // Should already have been set through layout of the parent().
         RenderObject* po = parent();
         for (; po && po != cb; po = po->parent())
             static_top += po->yPos();
@@ -1226,23 +1359,32 @@ void RenderBox::calcAbsoluteVertical()
     if (style()->hidesOverflow() && m_height > h+pab)
         m_height = h+pab;
     
+    // Do not allow the height to be negative.  This can happen when someone specifies both top and bottom
+    // but the containing block height is less than top, e.g., top:20px, bottom:0, containing block height 16.
+    m_height = kMax(0, m_height);
+    
     m_marginTop = mt;
     m_marginBottom = mb;
-    m_y = t + mt;
+    m_y = t + mt + cb->borderTop();
     
 //    printf("v: h=%d, t=%d, b=%d, mt=%d, mb=%d, m_y=%d\n",h,t,b,mt,mb,m_y);
 
 }
 
 
-int RenderBox::lowestPosition(bool includeOverflowInterior) const
+int RenderBox::lowestPosition(bool includeOverflowInterior, bool includeSelf) const
 {
-    return m_height;
+    return includeSelf ? m_height : 0;
 }
 
-int RenderBox::rightmostPosition(bool includeOverflowInterior) const
+int RenderBox::rightmostPosition(bool includeOverflowInterior, bool includeSelf) const
 {
-    return m_width;
+    return includeSelf ? m_width : 0;
+}
+
+int RenderBox::leftmostPosition(bool includeOverflowInterior, bool includeSelf) const
+{
+    return includeSelf ? 0 : m_width;
 }
 
 #undef DEBUG_LAYOUT
