@@ -30,10 +30,16 @@
 #include "render_text.h"
 #include "render_image.h"
 #include "render_canvas.h"
+#include "xml/dom_docimpl.h"
 
 #include <kdebug.h>
 #include <assert.h>
 
+#if APPLE_CHANGES
+// For accessibility
+#include "KWQAccObjectCache.h" 
+#endif
+
 using namespace khtml;
 
 RenderContainer::RenderContainer(DOM::NodeImpl* node)
@@ -48,22 +54,22 @@ RenderContainer::~RenderContainer()
 {
 }
 
-void RenderContainer::detach(RenderArena* renderArena)
+void RenderContainer::detach()
 {
     if (continuation())
-        continuation()->detach(renderArena);
+        continuation()->detach();
     
     RenderObject* next;
     for(RenderObject* n = m_first; n; n = next ) {
         n->removeFromObjectLists();
         n->setParent(0);
         next = n->nextSibling();
-        n->detach(renderArena);
+        n->detach();
     }
     m_first = 0;
     m_last = 0;
 
-    RenderObject::detach(renderArena);
+    RenderObject::detach();
 }
 
 bool RenderContainer::canHaveChildren() const
@@ -84,10 +90,12 @@ void RenderContainer::addChild(RenderObject *newChild, RenderObject *beforeChild
         switch(newChild->style()->display()) {
         case INLINE:
         case BLOCK:
+        case INLINE_BLOCK:
         case LIST_ITEM:
         case RUN_IN:
         case COMPACT:
-        case MARKER:
+        case BOX:
+        case INLINE_BOX:
         case TABLE:
         case INLINE_TABLE:
         case TABLE_COLUMN:
@@ -130,16 +138,15 @@ void RenderContainer::addChild(RenderObject *newChild, RenderObject *beforeChild
         RenderTable *table;
         if( !beforeChild )
             beforeChild = lastChild();
-        if( beforeChild && beforeChild->isAnonymousBox() && beforeChild->isTable() )
+        if( beforeChild && beforeChild->isAnonymous() && beforeChild->isTable() )
             table = static_cast<RenderTable *>(beforeChild);
         else {
             //kdDebug( 6040 ) << "creating anonymous table" << endl;
-            table = new (renderArena()) RenderTable(0 /* is anonymous */);
+            table = new (renderArena()) RenderTable(document() /* is anonymous */);
             RenderStyle *newStyle = new RenderStyle();
             newStyle->inheritFrom(style());
             newStyle->setDisplay(TABLE);
             table->setStyle(newStyle);
-            table->setIsAnonymousBox(true);
             addChild(table, beforeChild);
         }
         table->addChild(newChild);
@@ -147,34 +154,43 @@ void RenderContainer::addChild(RenderObject *newChild, RenderObject *beforeChild
 	// just add it...
 	insertChildNode(newChild, beforeChild);
     }
-    newChild->setNeedsLayoutAndMinMaxRecalc();
 }
 
 RenderObject* RenderContainer::removeChildNode(RenderObject* oldChild)
 {
     KHTMLAssert(oldChild->parent() == this);
 
-    // Keep our layer hierarchy updated.
-    oldChild->removeLayers(enclosingLayer());
-   
-    // if oldChild is the start or end of the selection, then clear the selection to
-    // avoid problems of invalid pointers
-
-    // ### This is not the "proper" solution... ideally the selection should be maintained
-    // based on DOM Nodes and a Range, which gets adjusted appropriately when nodes are
-    // deleted/inserted near etc. But this at least prevents crashes caused when the start
-    // or end of the selection is deleted and then accessed when the user next selects
-    // something.
-
-    if (oldChild->isSelectionBorder()) {
-        RenderObject *root = oldChild;
-        while (root && root->parent())
-            root = root->parent();
-        if (root->isCanvas()) {
-            static_cast<RenderCanvas*>(root)->clearSelection();
+    // So that we'll get the appropriate dirty bit set (either that a normal flow child got yanked or
+    // that a positioned child got yanked).  We also repaint, so that the area exposed when the child
+    // disappears gets repainted properly.
+    if (document()->renderer()) {
+        oldChild->setNeedsLayoutAndMinMaxRecalc();
+#ifdef INCREMENTAL_REPAINTING
+        oldChild->repaint();
+#endif
+        
+        // Keep our layer hierarchy updated.
+        oldChild->removeLayers(enclosingLayer());
+    
+        // if oldChild is the start or end of the selection, then clear the selection to
+        // avoid problems of invalid pointers
+    
+        // ### This is not the "proper" solution... ideally the selection should be maintained
+        // based on DOM Nodes and a Range, which gets adjusted appropriately when nodes are
+        // deleted/inserted near etc. But this at least prevents crashes caused when the start
+        // or end of the selection is deleted and then accessed when the user next selects
+        // something.
+    
+        if (oldChild->isSelectionBorder()) {
+            RenderObject *root = oldChild;
+            while (root && root->parent())
+                root = root->parent();
+            if (root->isCanvas()) {
+                static_cast<RenderCanvas*>(root)->clearSelection();
+            }
         }
     }
-
+    
     // remove the child
     if (oldChild->previousSibling())
         oldChild->previousSibling()->setNextSibling(oldChild->nextSibling());
@@ -190,7 +206,11 @@ RenderObject* RenderContainer::removeChildNode(RenderObject* oldChild)
     oldChild->setNextSibling(0);
     oldChild->setParent(0);
 
-    setNeedsLayoutAndMinMaxRecalc();
+#if APPLE_CHANGES
+    KWQAccObjectCache* cache = document()->getExistingAccObjectCache();
+    if (cache)
+        cache->childrenChanged(this);
+#endif
     
     return oldChild;
 }
@@ -198,37 +218,49 @@ RenderObject* RenderContainer::removeChildNode(RenderObject* oldChild)
 void RenderContainer::removeChild(RenderObject *oldChild)
 {
     removeChildNode(oldChild);
-    setNeedsLayout(true);
 }
 
 void RenderContainer::updatePseudoChild(RenderStyle::PseudoId type, RenderObject* child)
 {
-    RenderStyle* pseudo = style()->getPseudoStyle(type);
-    if (!pseudo || pseudo->display() == NONE) {
-        if (child && child->style()->styleType() == type)
-            // The child needs to be removed.
+    // In CSS2, before/after pseudo-content cannot nest.  Check this first.
+    if (style()->styleType() == RenderStyle::BEFORE || style()->styleType() == RenderStyle::AFTER)
+        return;
+    
+    RenderStyle* pseudo = getPseudoStyle(type);
+
+    // Whether or not we currently have generated content attached.
+    bool oldContentPresent = child && (child->style()->styleType() == type);
+
+    // Whether or not we now want generated content.  
+    bool newContentWanted = pseudo && pseudo->display() != NONE;
+
+    // For <q><p/></q>, if this object is the inline continuation of the <q>, we only want to generate
+    // :after content and not :before content.
+    if (type == RenderStyle::BEFORE && isInlineContinuation())
+        newContentWanted = false;
+
+    // Similarly, if we're the beginning of a <q>, and there's an inline continuation for our object,
+    // then we don't generate the :after content.
+    if (type == RenderStyle::AFTER && isRenderInline() && continuation())
+        newContentWanted = false;
+    
+    // If we don't want generated content any longer, or if we have generated content, but it's no longer
+    // identical to the new content data we want to build render objects for, then we nuke all
+    // of the old generated content.
+    if (!newContentWanted ||
+        (oldContentPresent && !child->style()->contentDataEquivalent(pseudo))) {
+        // Nuke the child. 
+        if (child && child->style()->styleType() == type) {
+            oldContentPresent = false;
             removeChild(child);
-        return; // If we have no pseudo-style or if the pseudo's display type is NONE, then we
-                // have no generated content.
+            child = (type == RenderStyle::BEFORE) ? firstChild() : lastChild();
+        }
     }
 
-    // FIXME: need to detect when :before/:after content has changed, in addition
-    // to detecting addition/removal.
-    if (child && child->style()->styleType() == type)
-        return; // Generated content is already added.  No need to add more.
-    
-    RenderObject* insertBefore = (type == RenderStyle::BEFORE) ? child : 0;
-        
-    // From the CSS2 specification:
-    // User agents must ignore the following properties with :before and :after
-    // pseudo-elements: 'position', 'float', list properties, and table properties.
-    // Basically we need to ensure that no RenderLayer gets made for generated
-    // content.
-    pseudo->setPosition(STATIC);
-    pseudo->setFloating(FNONE);
-    pseudo->setOverflow(OVISIBLE); // FIXME: Glazman's blog does this. Wacky.
-                                    // This property might need to be allowed if the
-                                    // generated content is a block.
+    // If we have no pseudo-style or if the pseudo's display type is NONE, then we
+    // have no generated content and can now return.
+    if (!newContentWanted)
+        return;
 
     if (isInlineFlow() && pseudo->display() != INLINE)
         // According to the CSS2 spec (the end of section 12.1), the only allowed
@@ -236,37 +268,67 @@ void RenderContainer::updatePseudoChild(RenderStyle::PseudoId type, RenderObject
         // determined that the pseudo is not display NONE, any display other than
         // inline should be mutated to INLINE.
         pseudo->setDisplay(INLINE);
+    
+    if (oldContentPresent) {
+        if (child && child->style()->styleType() == type) {
+            // We have generated content present still.  We want to walk this content and update our
+            // style information with the new pseudo style.
+            child->setStyle(pseudo);
 
+            // Note that if we ever support additional types of generated content (which should be way off
+            // in the future), this code will need to be patched.
+            for (RenderObject* genChild = child->firstChild(); genChild; genChild = genChild->nextSibling()) {
+                if (genChild->isText())
+                    // Generated text content is a child whose style also needs to be set to the pseudo
+                    // style.
+                    genChild->setStyle(pseudo);
+                else {
+                    // Images get an empty style that inherits from the pseudo.
+                    RenderStyle* style = new RenderStyle();
+                    style->inheritFrom(pseudo);
+                    genChild->setStyle(style);
+                }
+            }
+        }
+        return; // We've updated the generated content. That's all we needed to do.
+    }
+    
+    RenderObject* insertBefore = (type == RenderStyle::BEFORE) ? child : 0;
+
+    // Generated content consists of a single container that houses multiple children (specified
+    // by the content property).  This pseudo container gets the pseudo style set on it.
+    RenderObject* pseudoContainer = 0;
+    
     // Now walk our list of generated content and create render objects for every type
     // we encounter.
     for (ContentData* contentData = pseudo->contentData();
          contentData; contentData = contentData->_nextContent) {
+        if (!pseudoContainer)
+            pseudoContainer = RenderFlow::createAnonymousFlow(document(), pseudo); /* anonymous box */
+        
         if (contentData->contentType() == CONTENT_TEXT)
         {
-            RenderObject* po = RenderFlow::createFlow(0, pseudo, renderArena()); /* anonymous box */
-            
-            RenderText* t = new (renderArena()) RenderText(0 /*anonymous object */, contentData->contentText());
+            RenderText* t = new (renderArena()) RenderTextFragment(document() /*anonymous object */, contentData->contentText());
             t->setStyle(pseudo);
-            po->addChild(t);
-
-            // Add this after we've installed our text, so that addChild will be able to find the text
-            // inside the inline for e.g., first-letter styling.
-            addChild(po, insertBefore);
-            
-//            kdDebug() << DOM::DOMString(contentData->contentText()).string() << endl;
-
-            t->close();
-            po->close();
+            pseudoContainer->addChild(t);
         }
         else if (contentData->contentType() == CONTENT_OBJECT)
         {
-            RenderImage* po = new (renderArena()) RenderImage(0);
-            po->setStyle(pseudo);
-            po->setContentObject(contentData->contentObject());
-            addChild(po, insertBefore);
-            po->close();
+            RenderImage* img = new (renderArena()) RenderImage(document()); /* Anonymous object */
+            RenderStyle* style = new RenderStyle();
+            style->inheritFrom(pseudo);
+            img->setStyle(style);
+            img->setContentObject(contentData->contentObject());
+            pseudoContainer->addChild(img);
         }
     }
+
+    if (pseudoContainer) {
+        // Add the pseudo after we've installed all our content, so that addChild will be able to find the text
+        // inside the inline for e.g., first-letter styling.
+        addChild(pseudoContainer, insertBefore);
+        pseudoContainer->close();
+    }
 }
 
 
@@ -287,11 +349,22 @@ void RenderContainer::appendChildNode(RenderObject* newChild)
 
     setLastChild(newChild);
     
-    // Keep our layer hierarchy updated.
-    RenderLayer* layer = enclosingLayer();
-    newChild->addLayers(layer, newChild);
-
-    newChild->setNeedsLayoutAndMinMaxRecalc();
+    // Keep our layer hierarchy updated.  Optimize for the common case where we don't have any children
+    // and don't have a layer attached to ourselves.
+    if (newChild->firstChild() || newChild->layer()) {
+        RenderLayer* layer = enclosingLayer();
+        newChild->addLayers(layer, newChild);
+    }
+    
+    newChild->setNeedsLayoutAndMinMaxRecalc(); // Goes up the containing block hierarchy.
+    if (!normalChildNeedsLayout())
+        setChildNeedsLayout(true); // We may supply the static position for an absolute positioned child.
+    
+#if APPLE_CHANGES
+    KWQAccObjectCache* cache = document()->getExistingAccObjectCache();
+    if (cache)
+        cache->childrenChanged(this);
+#endif
 }
 
 void RenderContainer::insertChildNode(RenderObject* child, RenderObject* beforeChild)
@@ -302,7 +375,7 @@ void RenderContainer::insertChildNode(RenderObject* child, RenderObject* beforeC
     }
 
     KHTMLAssert(!child->parent());
-    while ( beforeChild->parent() != this && beforeChild->parent()->isAnonymousBox() )
+    while ( beforeChild->parent() != this && beforeChild->parent()->isAnonymousBlock() )
 	beforeChild = beforeChild->parent();
     KHTMLAssert(beforeChild->parent() == this);
 
@@ -322,6 +395,14 @@ void RenderContainer::insertChildNode(RenderObject* child, RenderObject* beforeC
     child->addLayers(layer, child);
 
     child->setNeedsLayoutAndMinMaxRecalc();
+    if (!normalChildNeedsLayout())
+        setChildNeedsLayout(true); // We may supply the static position for an absolute positioned child.
+    
+#if APPLE_CHANGES
+    KWQAccObjectCache* cache = document()->getExistingAccObjectCache();
+    if (cache)
+        cache->childrenChanged(this);
+#endif    
 }
 
 
@@ -346,7 +427,7 @@ void RenderContainer::removeLeftoverAnonymousBoxes()
     while( child ) {
 	RenderObject *next = child->nextSibling();
 	
-	if ( child->isRenderBlock() && child->isAnonymousBox() && !child->continuation() && !child->childrenInline() && !child->isTableCell() ) {
+	if ( child->isRenderBlock() && child->isAnonymousBlock() && !child->continuation() && !child->childrenInline() && !child->isTableCell() ) {
 	    RenderObject *firstAnChild = child->firstChild();
 	    RenderObject *lastAnChild = child->lastChild();
 	    if ( firstAnChild ) {
@@ -380,7 +461,7 @@ void RenderContainer::removeLeftoverAnonymousBoxes()
 		c->m_first = 0;
 		c->m_next = 0;
 	    }
-	    child->detach(renderArena());
+	    child->detach();
 	}
 	child = next;
     }
