@@ -92,14 +92,13 @@ public:
     void addIntrinsicMarginsIfAllowed(RenderStyle* _style);
     virtual bool canHaveIntrinsicMargins() const { return false; }
     int intrinsicMargin() const { return 2; }
-
-    virtual void setStyle(RenderStyle* _style);
 #endif
 
+    virtual void setStyle(RenderStyle *);
     virtual void updateFromElement();
 
     virtual void layout();
-    virtual short baselinePosition( bool ) const;
+    virtual short baselinePosition( bool, bool ) const;
 
     DOM::HTMLGenericFormElementImpl *element() const
     { return static_cast<DOM::HTMLGenericFormElementImpl*>(RenderObject::element()); }
@@ -133,7 +132,7 @@ public:
 #endif
 
     virtual const char *renderName() const { return "RenderButton"; }
-    virtual short baselinePosition( bool ) const;
+    virtual short baselinePosition( bool, bool ) const;
 
     // don't even think about making this method virtual!
     DOM::HTMLInputElementImpl* element() const
@@ -193,7 +192,11 @@ public:
 
     virtual void calcMinMaxWidth();
     virtual void updateFromElement();
-    virtual short baselinePosition( bool ) const;
+    virtual short baselinePosition( bool, bool ) const;
+#if APPLE_CHANGES
+    virtual void setStyle(RenderStyle *);
+#endif
+
 private:
     QString rawText();
 };
@@ -247,6 +250,7 @@ public:
 
     virtual const char *renderName() const { return "RenderLineEdit"; }
     virtual void updateFromElement();
+    virtual void setStyle(RenderStyle *);
 
     void select();
 
@@ -288,6 +292,8 @@ public:
     virtual const char *renderName() const { return "RenderFieldSet"; }
 
     virtual RenderObject* layoutLegend(bool relayoutChildren);
+
+    virtual void setStyle(RenderStyle* _style);
     
 protected:
     virtual void paintBoxDecorations(QPainter *p,int, int _y,
@@ -317,7 +323,13 @@ public:
     DOM::HTMLInputElementImpl *element() const
     { return static_cast<DOM::HTMLInputElementImpl*>(RenderObject::element()); }
 
+#if !APPLE_CHANGES
     KLineEdit* lineEdit() const { return m_edit; }
+#endif
+
+#if APPLE_CHANGES
+    void click();
+#endif
 
 public slots:
     virtual void slotClicked();
@@ -325,14 +337,14 @@ public slots:
     virtual void slotTextChanged(const QString &string);
 
 protected:
-    virtual void handleFocusOut();
-
     virtual bool isEditable() const { return true; }
 
-    bool m_clicked;
-    bool m_haveFocus;
+#if !APPLE_CHANGES
+    virtual void handleFocusOut();
+
     KLineEdit   *m_edit;
     QPushButton *m_button;
+#endif
 };
 
 
@@ -380,7 +392,7 @@ public:
     virtual const char *renderName() const { return "RenderSelect"; }
 
 #if APPLE_CHANGES
-    short baselinePosition( bool f ) const;
+    short baselinePosition( bool f, bool b ) const;
     int calcReplacedHeight() const { if (!m_useListBox) return intrinsicHeight(); return RenderFormElement::calcReplacedHeight(); }
     virtual bool canHaveIntrinsicMargins() const { return true; }
 #endif
@@ -393,6 +405,9 @@ public:
     bool selectionChanged() { return m_selectionChanged; }
     void setSelectionChanged(bool _selectionChanged) { m_selectionChanged = _selectionChanged; }
     virtual void updateFromElement();
+#if APPLE_CHANGES
+    virtual void setStyle(RenderStyle *);
+#endif
 
     void updateSelection();
 
@@ -402,6 +417,9 @@ public:
 protected:
     KListBox *createListBox();
     ComboBoxWidget *createComboBox();
+#if APPLE_CHANGES
+    void setWidgetWritingDirection();
+#endif
 
     unsigned  m_size;
     bool m_multiple;
@@ -435,12 +453,13 @@ class RenderTextArea : public RenderFormElement
 public:
     RenderTextArea(DOM::HTMLTextAreaElementImpl *element);
 
-    virtual void detach(RenderArena *arena);
+    virtual void detach();
 
     virtual const char *renderName() const { return "RenderTextArea"; }
     virtual void calcMinMaxWidth();
     virtual void close ( );
     virtual void updateFromElement();
+    virtual void setStyle(RenderStyle *);
 
     virtual bool isTextArea() const { return true; }
     
