@@ -20,7 +20,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: table_layout.cpp,v 1.13 2003/05/08 02:19:38 hyatt Exp $
+ * $Id: table_layout.cpp,v 1.17 2003/10/17 22:32:14 hyatt Exp $
  */
 #include "table_layout.h"
 #include "render_table.h"
@@ -173,8 +173,11 @@ int FixedTableLayout::calcWidthArray( int tableWidth )
 	section = table->foot;
     if ( section ) {
 	cCol = 0;
-	// get the first cell in the first row
-	child = section->firstChild()->firstChild();
+        // FIXME: Technically the first row could be in an arbitrary section (e.g., an nth section
+        // if the previous n-1 sections have no rows), so this check isn't good enough.
+        // get the first cell in the first row
+        RenderObject* firstRow = section->firstChild();
+        child = firstRow ? firstRow->firstChild() : 0;
 	while ( child ) {
 	    if ( child->isTableCell() ) {
 		RenderTableCell *cell = static_cast<RenderTableCell *>(child);
@@ -234,7 +237,7 @@ void FixedTableLayout::calcMinMaxWidth()
     // width is fixed. If table width is percent, we set maxWidth to
     // unlimited.
 
-    int bs = table->bordersAndSpacing();
+    int bs = table->bordersPaddingAndSpacing();
     
     int tableWidth = table->style()->width().type == Fixed ? table->style()->width().value - bs : 0;
     int mw = calcWidthArray( tableWidth ) + bs;
@@ -260,7 +263,7 @@ void FixedTableLayout::calcMinMaxWidth()
 
 void FixedTableLayout::layout()
 {
-    int tableWidth = table->width() - table->bordersAndSpacing();
+    int tableWidth = table->width() - table->bordersPaddingAndSpacing();
     int available = tableWidth;
     int nEffCols = table->numEffCols();
     int totalPercent = 0;
@@ -342,13 +345,13 @@ void FixedTableLayout::layout()
     }
     
     int pos = 0;
-    int spacing = table->cellSpacing();
+    int hspacing = table->hBorderSpacing();
     for ( int i = 0; i < nEffCols; i++ ) {
 #ifdef DEBUG_LAYOUT
 	qDebug("col %d: %d (width %d)", i, pos, calcWidth[i] );
 #endif
 	table->columnPos[i] = pos;
-	pos += calcWidth[i] + spacing;
+	pos += calcWidth[i] + hspacing;
     }
     table->columnPos[table->columnPos.size()-1] = pos;
 }
@@ -602,7 +605,7 @@ void AutoTableLayout::calcMinMaxWidth()
 
     maxWidth = kMax( maxWidth, spanMaxWidth );
     
-    int bs = table->bordersAndSpacing();
+    int bs = table->bordersPaddingAndSpacing();
     minWidth += bs;
     maxWidth += bs;
 
@@ -628,7 +631,7 @@ int AutoTableLayout::calcEffectiveWidth()
     int tMaxWidth = 0;
 
     unsigned int nEffCols = layoutStruct.size();
-    int spacing = table->cellSpacing();
+    int hspacing = table->hBorderSpacing();
 #ifdef DEBUG_LAYOUT
     qDebug("AutoTableLayout::calcEffectiveWidth for %d cols", nEffCols );
 #endif
@@ -650,8 +653,8 @@ int AutoTableLayout::calcEffectiveWidth()
 
 	int col = table->colToEffCol( cell->col() );
 	unsigned int lastCol = col;
-	int cMinWidth = cell->minWidth() + spacing;
-	int cMaxWidth = cell->maxWidth() + spacing;
+	int cMinWidth = cell->minWidth() + hspacing;
+	int cMaxWidth = cell->maxWidth() + hspacing;
 	int totalPercent = 0;
 	int minWidth = 0;
 	int maxWidth = 0;
@@ -681,16 +684,27 @@ int AutoTableLayout::calcEffectiveWidth()
 		haveVariable = true;
 		// fall through
 	    default:
-		layoutStruct[lastCol].effWidth = Length();
-		allColsArePercent = false;
+                // If the column is a percentage width, do not let the spanning cell overwrite the
+                // width value.  This caused a mis-rendering on amazon.com.
+                // Sample snippet:
+                // <table border=2 width=100%><
+                //   <tr><td>1</td><td colspan=2>2-3</tr>
+                //   <tr><td>1</td><td colspan=2 width=100%>2-3</td></tr>
+                // </table>
+                if (layoutStruct[lastCol].effWidth.type != Percent) {
+                    layoutStruct[lastCol].effWidth = Length();
+                    allColsArePercent = false;
+                }
+                else
+                    totalPercent += layoutStruct[lastCol].effWidth.value;
 		allColsAreFixed = false;
 	    }
 	    span -= table->spanOfEffCol( lastCol );
 	    minWidth += layoutStruct[lastCol].effMinWidth;
 	    maxWidth += layoutStruct[lastCol].effMaxWidth;
 	    lastCol++;
-	    cMinWidth -= spacing;
-	    cMaxWidth -= spacing;
+	    cMinWidth -= hspacing;
+	    cMaxWidth -= hspacing;
 	}
 #ifdef DEBUG_LAYOUT
 	qDebug("    colspan cell %p at effCol %d, span %d, type %d, value %d cmin=%d min=%d fixedwidth=%d", cell, col, cSpan, w.type, w.value, cMinWidth, minWidth, fixedWidth );
@@ -725,7 +739,7 @@ int AutoTableLayout::calcEffectiveWidth()
 			totalWidth -= layoutStruct[pos].effMaxWidth;
 			percentMissing -= percent;
 			if ( percent > 0 )
-			    layoutStruct[pos].effWidth = Length( percent, Percent );
+		            layoutStruct[pos].effWidth = Length( percent, Percent );
 			else
 			    layoutStruct[pos].effWidth = Length();
 		    }
@@ -756,6 +770,8 @@ int AutoTableLayout::calcEffectiveWidth()
 #endif
 		int maxw = maxWidth;
                 int minw = minWidth;
+                
+                // Give min to variable first, to fixed second, and to others third.
                 for ( unsigned int pos = col; maxw > 0 && pos < lastCol; pos++ ) {
 		    if ( layoutStruct[pos].width.type == Fixed && haveVariable && fixedWidth <= cMinWidth ) {
 			int w = QMAX( layoutStruct[pos].effMinWidth, layoutStruct[pos].width.value );
@@ -842,7 +858,7 @@ void AutoTableLayout::insertSpanCell( RenderTableCell *cell )
 void AutoTableLayout::layout()
 {
     // table layout based on the values collected in the layout structure.
-    int tableWidth = table->width() - table->bordersAndSpacing();
+    int tableWidth = table->width() - table->bordersPaddingAndSpacing();
     int available = tableWidth;
     int nEffCols = table->numEffCols();
 
@@ -1144,7 +1160,7 @@ void AutoTableLayout::layout()
 	qDebug("col %d: %d (width %d)", i, pos, layoutStruct[i].calcWidth );
 #endif
 	table->columnPos[i] = pos;
-	pos += layoutStruct[i].calcWidth + table->cellSpacing();
+	pos += layoutStruct[i].calcWidth + table->hBorderSpacing();
     }
     table->columnPos[table->columnPos.size()-1] = pos;
 
