@@ -24,20 +24,26 @@
  */
 
 #import "KWQPainter.h"
-#import "KWQWidget.h"
-#import "KWQFontMetrics.h"
-#import "KWQPixmap.h"
-#import "KWQPtrStack.h"
-#import "KWQPointArray.h"
-#import "KWQPaintDevice.h"
-#import "KWQPrinter.h"
 
 #import "KWQAssertions.h"
-
+#import "KWQExceptions.h"
+#import "KWQFontMetrics.h"
+#import "KWQKHTMLPart.h"
+#import "KWQPaintDevice.h"
+#import "KWQPixmap.h"
+#import "KWQPointArray.h"
+#import "KWQPrinter.h"
+#import "KWQPtrStack.h"
+#import "KWQWidget.h"
+#import "WebCoreGraphicsBridge.h"
 #import "WebCoreImageRenderer.h"
 #import "WebCoreTextRenderer.h"
 #import "WebCoreTextRendererFactory.h"
 
+// NSColor, NSBezierPath, NSGraphicsContext and WebCoreTextRenderer
+// calls in this file are all exception-safe, so we don't block
+// exceptions for those.
+
 struct QPState {
     QPState() : paintingDisabled(false) { }
     QFont font;
@@ -48,19 +54,25 @@ struct QPState {
 };
 
 struct QPainterPrivate {
-    QPainterPrivate() : textRenderer(0) { }
-    ~QPainterPrivate() { [textRenderer release]; }
+    QPainterPrivate() : textRenderer(0), focusRingPath(0), focusRingWidth(0), focusRingOffset(0),
+                        hasFocusRingColor(false) { }
+    ~QPainterPrivate() { [textRenderer release]; [focusRingPath release]; }
     QPState state;
     QPtrStack<QPState> stack;
     id <WebCoreTextRenderer> textRenderer;
     QFont textRendererFont;
+    NSBezierPath *focusRingPath;
+    int focusRingWidth;
+    int focusRingOffset;
+    bool hasFocusRingColor;
+    QColor focusRingColor;
 };
 
-QPainter::QPainter() : data(new QPainterPrivate), _isForPrinting(false), _usesInactiveTextBackgroundColor(false)
+QPainter::QPainter() : data(new QPainterPrivate), _isForPrinting(false), _usesInactiveTextBackgroundColor(false), _drawsFocusRing(true)
 {
 }
 
-QPainter::QPainter(bool forPrinting) : data(new QPainterPrivate), _isForPrinting(forPrinting), _usesInactiveTextBackgroundColor(false)
+QPainter::QPainter(bool forPrinting) : data(new QPainterPrivate), _isForPrinting(forPrinting), _usesInactiveTextBackgroundColor(false), _drawsFocusRing(true)
 {
 }
 
@@ -169,11 +181,9 @@ void QPainter::drawRect(int x, int y, int w, int h)
     if (data->state.paintingDisabled)
         return;
         
-    if (data->state.brush.style() != NoBrush) {
-        _setColorFromBrush();
-        
-        NSRectFillUsingOperation (NSMakeRect(x,y,w,h), NSCompositeSourceOver);
-    }
+    if (data->state.brush.style() != NoBrush)
+        _fillRect(x, y, w, h, data->state.brush.color());
+
     if (data->state.pen.style() != NoPen) {
         _setColorFromPen();
         NSFrameRect(NSMakeRect(x, y, w, h));
@@ -195,7 +205,7 @@ void QPainter::drawLine(int x1, int y1, int x2, int y2)
 {
     if (data->state.paintingDisabled)
         return;
-        
+
     PenStyle penStyle = data->state.pen.style();
     if (penStyle == NoPen)
         return;
@@ -251,6 +261,7 @@ void QPainter::drawLine(int x1, int y1, int x2, int y2)
     }
 
     _setColorFromPen();
+    
     NSGraphicsContext *graphicsContext = [NSGraphicsContext currentContext];
     BOOL flag = [graphicsContext shouldAntialias];
     [graphicsContext setShouldAntialias: NO];
@@ -259,12 +270,12 @@ void QPainter::drawLine(int x1, int y1, int x2, int y2)
         // Do a rect fill of our endpoints.  This ensures we always have the
         // appearance of being a border.  We then draw the actual dotted/dashed line.
         if (x1 == x2) {
-            NSRectFill(NSMakeRect(p1.x-width/2, p1.y-width, width, width));
-            NSRectFill(NSMakeRect(p2.x-width/2, p2.y, width, width));
+            _fillRect(p1.x-width/2, p1.y-width, width, width, data->state.pen.color());
+            _fillRect(p2.x-width/2, p2.y, width, width, data->state.pen.color());
         }
         else {
-            NSRectFill(NSMakeRect(p1.x-width, p1.y-width/2, width, width));
-            NSRectFill(NSMakeRect(p2.x, p2.y-width/2, width, width));
+            _fillRect(p1.x-width, p1.y-width/2, width, width, data->state.pen.color());
+            _fillRect(p2.x, p2.y-width/2, width, width, data->state.pen.color());
         }
         
         // Example: 80 pixels with a width of 30 pixels.
@@ -326,6 +337,7 @@ void QPainter::drawEllipse(int x, int y, int w, int h)
     }
     if (data->state.pen.style() != NoPen) {
         _setColorFromPen();
+        [path setLineWidth:data->state.pen.width()];
         [path stroke];
     }
 }
@@ -352,6 +364,7 @@ void QPainter::drawArc (int x, int y, int w, int h, int a, int alen)
                                       clockwise:YES];
     
         _setColorFromPen();
+        [path setLineWidth:data->state.pen.width()];
         [path stroke];
         [path release];
     }
@@ -405,6 +418,7 @@ void QPainter::_drawPoints (const QPointArray &_points, bool winding, int index,
 
     if (data->state.pen.style() != NoPen) {
         _setColorFromPen();
+        [path setLineWidth:data->state.pen.width()];
         [path stroke];
     }
     
@@ -433,9 +447,14 @@ void QPainter::drawPixmap( int x, int y, const QPixmap &pixmap,
         sw = pixmap.width();
     if (sh == -1)
         sh = pixmap.height();
+
+    NSRect inRect = NSMakeRect(x, y, sw, sh);
+    NSRect fromRect = NSMakeRect(sx, sy, sw, sh);
     
-    [pixmap.imageRenderer beginAnimationInRect:NSMakeRect(x, y, sw, sh)
-                                      fromRect:NSMakeRect(sx, sy, sw, sh)];
+    KWQ_BLOCK_EXCEPTIONS;
+    [pixmap.imageRenderer drawImageInRect:inRect
+                                      fromRect:fromRect];
+    KWQ_UNBLOCK_EXCEPTIONS;
 }
 
 void QPainter::drawTiledPixmap( int x, int y, int w, int h,
@@ -444,7 +463,9 @@ void QPainter::drawTiledPixmap( int x, int y, int w, int h,
     if (data->state.paintingDisabled)
         return;
     
+    KWQ_BLOCK_EXCEPTIONS;
     [pixmap.imageRenderer tileInRect:NSMakeRect(x, y, w, h) fromPoint:NSMakePoint(sx, sy)];
+    KWQ_UNBLOCK_EXCEPTIONS;
 }
 
 void QPainter::_updateRenderer(NSString **families)
@@ -452,10 +473,12 @@ void QPainter::_updateRenderer(NSString **families)
     if (data->textRenderer == 0 || data->state.font != data->textRendererFont) {
         data->textRendererFont = data->state.font;
         id <WebCoreTextRenderer> oldRenderer = data->textRenderer;
+	KWQ_BLOCK_EXCEPTIONS;
         data->textRenderer = [[[WebCoreTextRendererFactory sharedFactory]
             rendererWithFont:data->textRendererFont.getNSFont()
             usingPrinterFont:data->textRendererFont.isPrinterFont()] retain];
         [oldRenderer release];
+	KWQ_UNBLOCK_EXCEPTIONS;
     }
 }
     
@@ -469,26 +492,24 @@ void QPainter::drawText(int x, int y, int, int, int alignmentFlags, const QStrin
     CREATE_FAMILY_ARRAY(data->state.font, families);
 
     _updateRenderer(families);
-    
+
     const UniChar* str = (const UniChar*)qstring.unicode();
+
+    WebCoreTextRun run;
+    WebCoreInitializeTextRun(&run, str, qstring.length(), 0, qstring.length());
+    
+    WebCoreTextStyle style;
+    WebCoreInitializeEmptyTextStyle(&style);
+    style.textColor = data->state.pen.color().getNSColor();
+    style.families = families;
+    
     if (alignmentFlags & Qt::AlignRight)
-        x -= ROUND_TO_INT([data->textRenderer floatWidthForCharacters:(const UniChar *)str stringLength:qstring.length() fromCharacterPosition:0 numberOfCharacters:qstring.length() withPadding: 0 applyRounding:YES attemptFontSubstitution: YES widths: 0 letterSpacing: 0 wordSpacing: 0 smallCaps: false fontFamilies: families]);
+        x -= ROUND_TO_INT([data->textRenderer floatWidthForRun:&run style:&style widths:0]);
      
-    [data->textRenderer drawCharacters:str stringLength:qstring.length()
-        fromCharacterPosition:0 
-        toCharacterPosition:qstring.length() 
-        atPoint:NSMakePoint(x, y)
-        withPadding: 0
-        withTextColor:data->state.pen.color().getNSColor() 
-        backgroundColor:nil
-        rightToLeft: false
-        letterSpacing: 0
-        wordSpacing: 0
-        smallCaps: false
-        fontFamilies: families];
+    [data->textRenderer drawRun:&run style:&style atPoint:NSMakePoint(x, y)];
 }
 
-void QPainter::drawText(int x, int y, const QChar *str, int len, int from, int to, int toAdd, const QColor &backgroundColor, QPainter::TextDirection d, int letterSpacing, int wordSpacing, bool smallCaps)
+void QPainter::drawText(int x, int y, const QChar *str, int len, int from, int to, int toAdd, const QColor &backgroundColor, QPainter::TextDirection d, bool visuallyOrdered, int letterSpacing, int wordSpacing, bool smallCaps)
 {
     if (data->state.paintingDisabled || len <= 0)
         return;
@@ -499,19 +520,59 @@ void QPainter::drawText(int x, int y, const QChar *str, int len, int from, int t
     
     _updateRenderer(families);
 
-    [data->textRenderer
-    	drawCharacters:(const UniChar *)str stringLength:len
-        fromCharacterPosition:from 
-        toCharacterPosition:to 
-        atPoint:NSMakePoint(x, y)
-        withPadding: toAdd
-        withTextColor:data->state.pen.color().getNSColor() 
-        backgroundColor:backgroundColor.isValid() ? backgroundColor.getNSColor() : nil
-        rightToLeft: d == RTL ? true : false
-        letterSpacing: letterSpacing
-        wordSpacing: wordSpacing
-        smallCaps: smallCaps
-        fontFamilies: families];
+    if (from < 0)
+        from = 0;
+    if (to < 0)
+        to = len;
+        
+    WebCoreTextRun run;
+    WebCoreInitializeTextRun(&run, (const UniChar *)str, len, from, to);    
+    WebCoreTextStyle style;
+    WebCoreInitializeEmptyTextStyle(&style);
+    style.textColor = data->state.pen.color().getNSColor();
+    style.backgroundColor = backgroundColor.isValid() ? backgroundColor.getNSColor() : nil;
+    style.rtl = d == RTL ? true : false;
+    style.visuallyOrdered = visuallyOrdered;
+    style.letterSpacing = letterSpacing;
+    style.wordSpacing = wordSpacing;
+    style.smallCaps = smallCaps;
+    style.families = families;
+    style.padding = toAdd;
+    
+    [data->textRenderer drawRun:&run style:&style atPoint:NSMakePoint(x, y)];
+}
+
+void QPainter::drawHighlightForText(int x, int y, const QChar *str, int len, int from, int to, int toAdd, const QColor &backgroundColor, QPainter::TextDirection d, bool visuallyOrdered, int letterSpacing, int wordSpacing, bool smallCaps)
+{
+    if (data->state.paintingDisabled || len <= 0)
+        return;
+
+    // Avoid allocations, use stack array to pass font families.  Normally these
+    // css fallback lists are small <= 3.
+    CREATE_FAMILY_ARRAY(data->state.font, families);
+    
+    _updateRenderer(families);
+
+    if (from < 0)
+        from = 0;
+    if (to < 0)
+        to = len;
+        
+    WebCoreTextRun run;
+    WebCoreInitializeTextRun(&run, (const UniChar *)str, len, from, to);    
+    WebCoreTextStyle style;
+    WebCoreInitializeEmptyTextStyle(&style);
+    style.textColor = data->state.pen.color().getNSColor();
+    style.backgroundColor = backgroundColor.isValid() ? backgroundColor.getNSColor() : nil;
+    style.rtl = d == RTL ? true : false;
+    style.visuallyOrdered = visuallyOrdered;
+    style.letterSpacing = letterSpacing;
+    style.wordSpacing = wordSpacing;
+    style.smallCaps = smallCaps;
+    style.families = families;
+    style.padding = toAdd;
+    
+    [data->textRenderer drawHighlightForRun:&run style:&style atPoint:NSMakePoint(x, y)];
 }
 
 void QPainter::drawLineForText(int x, int y, int yOffset, int width)
@@ -533,15 +594,19 @@ QColor QPainter::selectedTextBackgroundColor() const
     return QColor((int)(255 * [color redComponent]), (int)(255 * [color greenComponent]), (int)(255 * [color blueComponent]));
 }
 
+void QPainter::_fillRect(float x, float y, float w, float h, const QColor& col)
+{
+    [col.getNSColor() set];
+    NSRectFillUsingOperation(NSMakeRect(x,y,w,h), NSCompositeSourceOver);
+}
+
 void QPainter::fillRect(int x, int y, int w, int h, const QBrush &brush)
 {
     if (data->state.paintingDisabled)
         return;
-    
-    if (brush.style() == SolidPattern) {
-        [brush.color().getNSColor() set];
-        NSRectFill(NSMakeRect(x, y, w, h));
-    }
+
+    if (brush.style() == SolidPattern)
+        _fillRect(x, y, w, h, brush.color());
 }
 
 void QPainter::addClip(const QRect &rect)
@@ -567,3 +632,116 @@ bool QPainter::paintingDisabled() const
 {
     return data->state.paintingDisabled;
 }
+
+void QPainter::beginTransparencyLayer(float opacity)
+{
+    CGContextRef context = (CGContextRef)([[NSGraphicsContext currentContext] graphicsPort]);
+    CGContextSetAlpha(context, opacity);
+    CGContextBeginTransparencyLayer(context, 0);
+}
+
+void QPainter::endTransparencyLayer()
+{
+    CGContextRef context = (CGContextRef)([[NSGraphicsContext currentContext] graphicsPort]);
+    CGContextEndTransparencyLayer(context);
+}
+
+void QPainter::setShadow(int x, int y, int blur, const QColor& color)
+{
+    // Check for an invalid color, as this means that the color was not set for the shadow
+    // and we should therefore just use the default shadow color.
+    CGContextRef context = (CGContextRef)([[NSGraphicsContext currentContext] graphicsPort]);
+    if (!color.isValid())
+        CGContextSetShadow(context, CGSizeMake(x,-y), blur); // y is flipped.
+    else {
+        NSColor* deviceColor = [color.getNSColor() colorUsingColorSpaceName: @"NSDeviceRGBColorSpace"];
+        float red = [deviceColor redComponent];
+        float green = [deviceColor greenComponent];
+        float blue = [deviceColor blueComponent];
+        float alpha = [deviceColor alphaComponent];
+        const float components[] = { red, green, blue, alpha };
+        
+        CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
+        CGColorRef color = CGColorCreate(colorSpace, components);
+        CGColorSpaceRelease(colorSpace);
+        CGContextSetShadowWithColor(context,
+                                    CGSizeMake(x,-y), // y is flipped.
+                                    blur, 
+                                    color);
+        CGColorRelease(color);
+    }
+}
+
+void QPainter::clearShadow()
+{
+    CGContextRef context = (CGContextRef)([[NSGraphicsContext currentContext] graphicsPort]);
+    CGContextSetShadowWithColor(context, CGSizeZero, 0, NULL);
+}
+
+void QPainter::initFocusRing(int width, int offset)
+{
+    if (!_drawsFocusRing)
+        return;
+
+    clearFocusRing();
+    data->focusRingWidth = width;
+    data->hasFocusRingColor = false;
+    data->focusRingOffset = offset;
+    data->focusRingPath = [[NSBezierPath alloc] init];
+    [data->focusRingPath setWindingRule:NSNonZeroWindingRule];
+}
+
+void QPainter::initFocusRing(int width, int offset, const QColor &color)
+{
+    if (!_drawsFocusRing)
+        return;
+
+    initFocusRing(width, offset);
+    data->hasFocusRingColor = true;
+    data->focusRingColor = color;
+}
+
+void QPainter::addFocusRingRect(int x, int y, int width, int height)
+{
+    if (!_drawsFocusRing)
+        return;
+
+    ASSERT(data->focusRingPath);
+    NSRect rect = NSMakeRect(x, y, width, height);
+    int offset = (data->focusRingWidth-1)/2 + data->focusRingOffset;
+    rect = NSInsetRect(rect, -offset, -offset);
+    [data->focusRingPath appendBezierPathWithRect:rect];
+}
+
+void QPainter::drawFocusRing()
+{
+    if (!_drawsFocusRing)
+        return;
+    
+    ASSERT(data->focusRingPath);
+    if (data->state.paintingDisabled)
+        return;
+
+    if ([data->focusRingPath elementCount] == 0) {
+        ERROR("Request to draw focus ring with no control points");
+        return;
+    }
+    
+    NSRect bounds = [data->focusRingPath bounds];
+    if (!NSIsEmptyRect(bounds)) {
+        int radius = (data->focusRingWidth-1)/2;
+        NSColor *color = data->hasFocusRingColor ? data->focusRingColor.getNSColor() : nil;
+        [NSGraphicsContext saveGraphicsState];
+        [[WebCoreGraphicsBridge sharedBridge] setFocusRingStyle:NSFocusRingOnly radius:radius color:color];
+        [data->focusRingPath fill];
+        [NSGraphicsContext restoreGraphicsState];   
+    }
+}
+
+void QPainter::clearFocusRing()
+{
+    if (data->focusRingPath) {
+        [data->focusRingPath release];
+        data->focusRingPath = nil;
+    }
+}
