@@ -29,12 +29,15 @@
 #import "dom_node.h"
 #import "dom_docimpl.h"
 #import "dom_nodeimpl.h"
-#import "html_formimpl.h"
 #import "html_documentimpl.h"
+#import "html_formimpl.h"
+#import "html_imageimpl.h"
 #import "htmlattrs.h"
 #import "htmltags.h"
 #import "khtml_part.h"
 #import "khtmlview.h"
+#import "kjs_proxy.h"
+#import "kjs_window.h"
 #import "loader.h"
 #import "render_frames.h"
 #import "render_image.h"
@@ -42,8 +45,9 @@
 #import "render_canvas.h"
 #import "render_style.h"
 #import "render_replaced.h"
-using khtml::RenderWidget;
 
+#import <JavaScriptCore/jni_jsobject.h>
+#import <JavaScriptCore/object.h>
 #import <JavaScriptCore/property_map.h>
 
 #import "KWQAssertions.h"
@@ -56,10 +60,12 @@ using khtml::RenderWidget;
 #import "KWQRenderTreeDebug.h"
 #import "KWQView.h"
 #import "KWQPrinter.h"
+#import "KWQAccObjectCache.h"
 
 #import "WebCoreDOMPrivate.h"
 #import "WebCoreImageRenderer.h"
 #import "WebCoreTextRendererFactory.h"
+#import "WebCoreViewFactory.h"
 #import "WebCoreSettings.h"
 
 #import <AppKit/NSView.h>
@@ -70,16 +76,20 @@ using DOM::NodeImpl;
 
 using khtml::Decoder;
 using khtml::parseURL;
+using khtml::RenderCanvas;
 using khtml::RenderImage;
 using khtml::RenderObject;
 using khtml::RenderPart;
 using khtml::RenderStyle;
-using khtml::RenderCanvas;
+using khtml::RenderWidget;
 
 using KJS::SavedProperties;
+using KJS::SavedBuiltins;
 
 using KParts::URLArgs;
 
+using KJS::Bindings::RootObject;
+
 NSString *WebCoreElementFrameKey = 		@"WebElementFrame";
 NSString *WebCoreElementImageAltStringKey = 	@"WebElementImageAltString";
 NSString *WebCoreElementImageKey = 		@"WebElementImage";
@@ -91,12 +101,31 @@ NSString *WebCoreElementLinkTargetFrameKey =	@"WebElementTargetFrame";
 NSString *WebCoreElementLinkLabelKey = 		@"WebElementLinkLabel";
 NSString *WebCoreElementLinkTitleKey = 		@"WebElementLinkTitle";
 NSString *WebCoreElementNameKey = 		@"WebElementName";
+NSString *WebCoreElementTitleKey = 		@"WebCoreElementTitle"; // not in WebKit API for now, could be in API some day
 
-NSString *WebCorePageCacheStateKey =               @"WebCorePageCacheState";
+NSString *WebCorePageCacheStateKey =            @"WebCorePageCacheState";
+
+static RootObject *rootForView(void *v)
+{
+    NSView *aView = (NSView *)v;
+    WebCoreBridge *aBridge = [[WebCoreViewFactory sharedFactory] bridgeForView:aView];
+    if (aBridge) {
+        KWQKHTMLPart *part = [aBridge part];
+        RootObject *root = new RootObject(v);    // The root gets deleted by JavaScriptCore.
+        
+        root->setRootObjectImp (static_cast<KJS::ObjectImp *>(KJS::Window::retrieveWindow(part)));
+        root->setInterpreter (KJSProxy::proxy(part)->interpreter());
+        part->addPluginRootObject (root);
+            
+        return root;
+    }
+    return 0;
+}
 
 @implementation WebCoreBridge
 
 static bool initializedObjectCacheSize = FALSE;
+static bool initializedKJS = FALSE;
 
 - init
 {
@@ -110,6 +139,11 @@ static bool initializedObjectCacheSize = FALSE;
         initializedObjectCacheSize = TRUE;
     }
     
+    if (!initializedKJS) {
+        KJS::Bindings::RootObject::setFindRootObjectForNativeHandleFunction (rootForView);
+        initializedKJS = TRUE;
+    }
+    
     _shouldCreateRenderers = YES;
     
     return self;
@@ -158,7 +192,13 @@ static bool initializedObjectCacheSize = FALSE;
     _part->setParent([parent part]);
 }
 
-- (void)openURL:(NSString *)URL reload:(BOOL)reload contentType:(NSString *)contentType refresh:(NSString *)refresh lastModified:(NSDate *)lastModified pageCache:(NSDictionary *)pageCache
+- (void)provisionalLoadStarted
+{
+    _part->provisionalLoadStarted();
+}
+
+
+- (void)openURL:(NSURL *)URL reload:(BOOL)reload contentType:(NSString *)contentType refresh:(NSString *)refresh lastModified:(NSDate *)lastModified pageCache:(NSDictionary *)pageCache
 {
     if (pageCache) {
         KWQPageState *state = [pageCache objectForKey:WebCorePageCacheStateKey];
@@ -176,7 +216,7 @@ static bool initializedObjectCacheSize = FALSE;
     _part->browserExtension()->setURLArgs(args);
 
     // opening the URL
-    if (_part->didOpenURL([URL cString])) {
+    if (_part->didOpenURL(URL)) {
         // things we have to set up after calling didOpenURL
         if (refresh) {
             _part->addMetaData("http-refresh", QString::fromNSString(refresh));
@@ -212,9 +252,9 @@ static bool initializedObjectCacheSize = FALSE;
     _part->closeURL();
 }
 
-- (void)didNotOpenURL:(NSString *)URL
+- (void)didNotOpenURL:(NSURL *)URL
 {
-    _part->didNotOpenURL(QString::fromNSString(URL));
+    _part->didNotOpenURL(KURL(URL).url());
 }
 
 - (void)saveDocumentState
@@ -249,9 +289,9 @@ static bool initializedObjectCacheSize = FALSE;
     }
 }
 
-- (void)scrollToAnchorWithURL:(NSString *)URL
+- (void)scrollToAnchorWithURL:(NSURL *)URL
 {
-    _part->scrollToAnchor([URL cString]);
+    _part->scrollToAnchor(KURL(URL).url().latin1());
 }
 
 - (BOOL)saveDocumentToPageCache
@@ -261,6 +301,9 @@ static bool initializedObjectCacheSize = FALSE;
         return NO;
     }
     
+    if (!doc->view()) {
+        return NO;
+    }
     _part->clearTimers();
 
     SavedProperties *windowProperties = new SavedProperties;
@@ -269,12 +312,14 @@ static bool initializedObjectCacheSize = FALSE;
     SavedProperties *locationProperties = new SavedProperties;
     _part->saveLocationProperties(locationProperties);
     
+    SavedBuiltins *interpreterBuiltins = new SavedBuiltins;
+    _part->saveInterpreterBuiltins(*interpreterBuiltins);
+
     KWQPageState *pageState = [[[KWQPageState alloc] initWithDocument:doc
                                                                   URL:_part->m_url
                                                      windowProperties:windowProperties
-                                                   locationProperties:locationProperties] autorelease];
-
-
+                                                   locationProperties:locationProperties
+				                  interpreterBuiltins:interpreterBuiltins] autorelease];
     [pageState setPausedActions: _part->pauseActions((const void *)pageState)];
 
     return [self saveDocumentToPageCache:pageState];
@@ -324,6 +369,15 @@ static bool initializedObjectCacheSize = FALSE;
 }
 
 - (void)deselectAll
+{
+    [self deselectText];
+    DocumentImpl *doc = _part->xmlDocImpl();
+    if (doc) {
+        doc->setFocusNode(0);
+    }
+}
+
+- (void)deselectText
 {
     _part->slotClearSelection();
 }
@@ -333,6 +387,14 @@ static bool initializedObjectCacheSize = FALSE;
     return _part->isFrameSet();
 }
 
+- (NSString *)styleSheetForPrinting
+{
+    if (!_part->settings()->shouldPrintBackgrounds()) {
+        return @"* { background-image: none !important; background-color: white !important;}";
+    }
+    return nil;
+}
+
 - (void)reapplyStylesForDeviceType:(WebCoreDeviceType)deviceType
 {
     _part->setMediaType(deviceType == WebCoreDeviceScreen ? "screen" : "print");
@@ -341,6 +403,9 @@ static bool initializedObjectCacheSize = FALSE;
         static QPaintDevice screen;
         static QPrinter printer;
     	doc->setPaintDevice(deviceType == WebCoreDeviceScreen ? &screen : &printer);
+        if (deviceType != WebCoreDeviceScreen) {
+            doc->setPrintStyleSheet(QString::fromNSString([self styleSheetForPrinting]));
+        }
     }
     return _part->reparseConfiguration();
 }
@@ -362,17 +427,23 @@ static BOOL nowPrinting(WebCoreBridge *self)
     }
 }
 
-- (void)forceLayout
+- (void)forceLayoutAdjustingViewSize:(BOOL)flag
 {
     [self _setupRootForPrinting:YES];
     _part->forceLayout();
+    if (flag) {
+        [self adjustViewSize];
+    }
     [self _setupRootForPrinting:NO];
 }
 
-- (void)forceLayoutForPageWidth:(float)pageWidth
+- (void)forceLayoutWithMinimumPageWidth:(float)minPageWidth maximumPageWidth:(float)maxPageWidth adjustingViewSize:(BOOL)flag
 {
     [self _setupRootForPrinting:YES];
-    _part->forceLayoutForPageWidth(pageWidth);
+    _part->forceLayoutWithPageWidthRange(minPageWidth, maxPageWidth);
+    if (flag) {
+        [self adjustViewSize];
+    }
     [self _setupRootForPrinting:NO];
 }
 
@@ -381,6 +452,11 @@ static BOOL nowPrinting(WebCoreBridge *self)
     _part->sendResizeEvent();
 }
 
+- (void)sendScrollEvent
+{
+    _part->sendScrollEvent();
+}
+
 - (void)drawRect:(NSRect)rect withPainter:(QPainter *)p
 {
     [self _setupRootForPrinting:YES];
@@ -396,22 +472,48 @@ static BOOL nowPrinting(WebCoreBridge *self)
 {
     QPainter painter(nowPrinting(self));
     painter.setUsesInactiveTextBackgroundColor(_part->usesInactiveTextBackgroundColor());
+    painter.setDrawsFocusRing(_part->showsFirstResponder());
     [self drawRect:rect withPainter:&painter];
 }
 
-- (void)adjustFrames:(NSRect)rect
+// Used by pagination code called from AppKit when a standalone web page is printed.
+- (NSArray*)computePageRectsWithPrintWidthScaleFactor:(float)printWidthScaleFactor printHeight:(float)printHeight
 {
-    // Ick!  khtml sets the frame size during layout and
-    // the frame origins during drawing!  So we have to 
-    // layout and do a draw with rendering disabled to
-    // correctly adjust the frames.
-    [self forceLayout];
-    QPainter painter(nowPrinting(self));
-    painter.setPaintingDisabled(YES);
-    [self drawRect:rect withPainter:&painter];
+    [self _setupRootForPrinting:YES];
+    NSMutableArray* pages = [NSMutableArray arrayWithCapacity:5];
+    if (printWidthScaleFactor == 0 || printHeight == 0)
+        return pages;
+	
+    if (!_part || !_part->xmlDocImpl() || !_part->view()) return pages;
+    RenderCanvas* root = static_cast<khtml::RenderCanvas *>(_part->xmlDocImpl()->renderer());
+    if (!root) return pages;
+    
+    KHTMLView* view = _part->view();
+    NSView* documentView = view->getDocumentView();
+    if (!documentView)
+        return pages;
+	
+    float currPageHeight = printHeight;
+    float docHeight = root->layer()->height();
+    float docWidth = root->layer()->width();
+    float printWidth = docWidth/printWidthScaleFactor;
+    
+    // We need to give the part the opportunity to adjust the page height at each step.
+    for (float i = 0; i < docHeight; i += currPageHeight) {
+        float proposedBottom = kMin(docHeight, i + printHeight);
+        _part->adjustPageHeight(&proposedBottom, i, proposedBottom, i);
+        currPageHeight = kMax(1.0f, proposedBottom - i);
+        for (float j = 0; j < docWidth; j += printWidth) {
+            NSValue* val = [NSValue valueWithRect: NSMakeRect(j, i, printWidth, currPageHeight)];
+            [pages addObject: val];
+        }
+    }
+    [self _setupRootForPrinting:NO];
+    
+    return pages;
 }
 
-// Vertical pagination hook from AppKit
+// This is to support the case where a webview is embedded in the view that's being printed
 - (void)adjustPageHeightNew:(float *)newBottom top:(float)oldTop bottom:(float)oldBottom limit:(float)bottomLimit
 {
     [self _setupRootForPrinting:YES];
@@ -494,6 +596,8 @@ static BOOL nowPrinting(WebCoreBridge *self)
         _renderPart->setWidget(_part->view());
         // Now the render part owns the view, so we don't any more.
     }
+
+    _part->view()->initScrollBars();
 }
 
 - (void)mouseDown:(NSEvent *)event
@@ -516,16 +620,23 @@ static BOOL nowPrinting(WebCoreBridge *self)
     _part->mouseMoved(event);
 }
 
+- (BOOL)sendContextMenuEvent:(NSEvent *)event
+{
+    return _part->sendContextMenuEvent(event);
+}
+
 - (id <WebDOMElement>)elementForView:(NSView *)view
 {
-    // FIXME: implemetented currently for only a subset of the KWQ widgets
+    // FIXME: implemented currently for only a subset of the KWQ widgets
     if ([view conformsToProtocol:@protocol(KWQWidgetHolder)]) {
-        QWidget *widget = [(NSView <KWQWidgetHolder> *)view widget];
-        NodeImpl *node = static_cast<const RenderWidget *>(widget->eventFilterObject())->element();
-        return [WebCoreDOMElement elementWithImpl:static_cast<ElementImpl *>(node)];
-    } else {
-        return nil;
+        NSView <KWQWidgetHolder> *widgetHolder = view;
+        QWidget *widget = [widgetHolder widget];
+        if (widget != nil) {
+            NodeImpl *node = static_cast<const RenderWidget *>(widget->eventFilterObject())->element();
+            return [WebCoreDOMElement elementWithImpl:static_cast<ElementImpl *>(node)];
+        }
     }
+    return nil;
 }
 
 static NSView *viewForElement(DOM::ElementImpl *elementImpl)
@@ -544,7 +655,7 @@ static HTMLInputElementImpl *inputElementFromDOMElement(id <WebDOMElement>elemen
 {
     ASSERT([(NSObject *)element isKindOfClass:[WebCoreDOMElement class]]);
     DOM::ElementImpl *domElement = [(WebCoreDOMElement *)element elementImpl];
-    if (idFromNode(domElement) == ID_INPUT) {
+    if (domElement && idFromNode(domElement) == ID_INPUT) {
         return static_cast<HTMLInputElementImpl *>(domElement);
     }
     return nil;
@@ -554,7 +665,7 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
 {
     ASSERT([(NSObject *)element isKindOfClass:[WebCoreDOMElement class]]);
     DOM::ElementImpl *domElement = [(WebCoreDOMElement *)element elementImpl];
-    if (idFromNode(domElement) == ID_FORM) {
+    if (domElement && idFromNode(domElement) == ID_FORM) {
         return static_cast<HTMLFormElementImpl *>(domElement);
     }
     return nil;
@@ -656,7 +767,22 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
     NSMutableDictionary *element = [NSMutableDictionary dictionary];
     [element setObject:[NSNumber numberWithBool:_part->isPointInsideSelection((int)point.x, (int)point.y)]
                 forKey:WebCoreElementIsSelectedKey];
-    
+
+    // Find the title in the nearest enclosing DOM node.
+    // For <area> tags in image maps, walk the tree for the <area>, not the <img> using it.
+    for (NodeImpl *titleNode = nodeInfo.innerNode(); titleNode; titleNode = titleNode->parentNode()) {
+        if (titleNode->isElementNode()) {
+            const DOMString title = static_cast<ElementImpl *>(titleNode)->getAttribute(ATTR_TITLE);
+            if (!title.isNull()) {
+                // We found a node with a title.
+                QString titleText = title.string();
+                titleText.replace('\\', _part->backslashAsCurrencySymbol());
+                [element setObject:titleText.getNSString() forKey:WebCoreElementTitleKey];
+                break;
+            }
+        }
+    }
+
     NodeImpl *URLNode = nodeInfo.URLElement();
     if (URLNode) {
         ElementImpl *e = static_cast<ElementImpl *>(URLNode);
@@ -670,22 +796,17 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
         
         DOMString link = e->getAttribute(ATTR_HREF);
         if (!link.isNull()) {
-            // Look for the first #text node to use as a label.
-            NodeImpl *labelParent = e;
-            while (labelParent->hasChildNodes()){
-                NodeImpl *childNode = labelParent->firstChild();
-                unsigned short type = childNode->nodeType();
-                if (type == Node::TEXT_NODE){
-                    DOMStringImpl *dv = childNode->nodeValue().implementation();
-                    if (dv){
-                        NSString *value = [NSString stringWithCharacters: (const unichar *)dv->s length: dv->l];
-                        [element setObject:value forKey:WebCoreElementLinkLabelKey];
-                        break;
-                    }
+            if (e->firstChild()) {
+                Range r(_part->document());
+                r.setStartBefore(e->firstChild());
+                r.setEndAfter(e->lastChild());
+                QString t = _part->text(r);
+                if (!t.isEmpty()) {
+                    [element setObject:t.getNSString() forKey:WebCoreElementLinkLabelKey];
                 }
-                labelParent = childNode;
             }
-            [element setObject:_part->xmlDocImpl()->completeURL(link.string()).getNSString() forKey:WebCoreElementLinkURLKey];
+            QString URLString = parseURL(link).string();
+            [element setObject:_part->xmlDocImpl()->completeURL(URLString).getNSString() forKey:WebCoreElementLinkURLKey];
         }
         
         DOMString target = e->getAttribute(ATTR_TARGET);
@@ -698,36 +819,44 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
     }
 
     NodeImpl *node = nodeInfo.innerNonSharedNode();
-    if (node && isImage(node)){
-        ElementImpl *i = static_cast<ElementImpl*>(node);
-        DOMString attr = i->getAttribute(ATTR_SRC);
-        if (attr.isEmpty()) {
-            // Look for the URL in the DATA attribute of the OBJECT tag.
-            attr = i->getAttribute(ATTR_DATA);
-        }
-
-        if (!attr.isEmpty()) {
-            [element setObject:_part->xmlDocImpl()->completeURL(attr.string()).getNSString() forKey:WebCoreElementImageURLKey];
-        }
+    if (node && node->renderer() && node->renderer()->isImage()) {
         
-        DOMString alt = i->getAttribute(ATTR_ALT);
-        if (!alt.isNull()) {
-            QString altText = alt.string();
-            altText.replace('\\', _part->backslashAsCurrencySymbol());
-            [element setObject:altText.getNSString() forKey:WebCoreElementImageAltStringKey];
-        }
-        
-        RenderImage *r = (RenderImage *)node->renderer();
-        if (r) {
+        RenderImage *r = static_cast<RenderImage *>(node->renderer());
+        NSImage *image = r->pixmap().image();
+        // Only return image information if there is an image.
+        if (image && !r->isDisplayingError()) {
+            [element setObject:r->pixmap().image() forKey:WebCoreElementImageKey];
+            
             int x, y;
             if (r->absolutePosition(x, y)) {
                 NSValue *rect = [NSValue valueWithRect:NSMakeRect(x, y, r->contentWidth(), r->contentHeight())];
                 [element setObject:rect forKey:WebCoreElementImageRectKey];
             }
             
-            NSImage *image = r->pixmap().image();
-            if (image) {
-                [element setObject:image forKey:WebCoreElementImageKey];
+            ElementImpl *i = static_cast<ElementImpl*>(node);
+    
+            // FIXME: Code copied from RenderImage::updateFromElement; should share.
+            DOMString attr;
+            if (idFromNode(i) == ID_OBJECT) {
+                attr = i->getAttribute(ATTR_DATA);
+            } else {
+                attr = i->getAttribute(ATTR_SRC);
+            }
+            if (!attr.isEmpty()) {
+                QString URLString = parseURL(attr).string();
+                [element setObject:_part->xmlDocImpl()->completeURL(URLString).getNSString() forKey:WebCoreElementImageURLKey];
+            }
+            
+            // FIXME: Code copied from RenderImage::updateFromElement; should share.
+            DOMString alt;
+            if (idFromNode(i) == ID_INPUT)
+                alt = static_cast<HTMLInputElementImpl *>(i)->altText();
+            else if (idFromNode(i) == ID_IMG)
+                alt = static_cast<HTMLImageElementImpl *>(i)->altText();
+            if (!alt.isNull()) {
+                QString altText = alt.string();
+                altText.replace('\\', _part->backslashAsCurrencySymbol());
+                [element setObject:altText.getNSString() forKey:WebCoreElementImageAltStringKey];
             }
         }
     }
@@ -752,9 +881,6 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
         return;
     }
     _part->setZoomFactor(newZoomFactor);
-    // setZoomFactor will trigger a timed layout, but we want to do the layout before
-    // we do any drawing. This takes care of that. Without this we redraw twice.
-    [self setNeedsLayout];
 }
 
 - (CFStringEncoding)textEncoding
@@ -764,12 +890,20 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
 
 - (NSView *)nextKeyView
 {
-    return _part->nextKeyView(0, KWQSelectingNext);
+    DocumentImpl *doc = _part->xmlDocImpl();
+    if (!doc) {
+        return nil;
+    }
+    return _part->nextKeyView(doc->focusNode(), KWQSelectingNext);
 }
 
 - (NSView *)previousKeyView
 {
-    return _part->nextKeyView(0, KWQSelectingPrevious);
+    DocumentImpl *doc = _part->xmlDocImpl();
+    if (!doc) {
+        return nil;
+    }
+    return _part->nextKeyView(doc->focusNode(), KWQSelectingPrevious);
 }
 
 - (NSView *)nextKeyViewInsideWebFrameViews
@@ -795,17 +929,21 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
 
 - (void)setSelectionFrom:(id<WebDOMNode>)start startOffset:(int)startOffset to:(id<WebDOMNode>)end endOffset:(int) endOffset
 {
-    _part->xmlDocImpl()->setSelection([(WebCoreDOMNode *)start impl], startOffset, [(WebCoreDOMNode *)end impl], endOffset);
+    WebCoreDOMNode *startNode = start;
+    WebCoreDOMNode *endNode = end;
+    _part->xmlDocImpl()->setSelection([startNode impl], startOffset, [endNode impl], endOffset);
 }
 
 - (NSAttributedString *)selectedAttributedString
 {
-    return KWQKHTMLPart::attributedString(_part->selectionStart(), _part->selectionStartOffset(), _part->selectionEnd(), _part->selectionEndOffset());
+    return _part->attributedString(_part->selectionStart(), _part->selectionStartOffset(), _part->selectionEnd(), _part->selectionEndOffset());
 }
 
-- (NSAttributedString *)attributedStringFrom:(id<WebDOMNode>)startNode startOffset:(int)startOffset to:(id<WebDOMNode>)endNode endOffset:(int)endOffset
+- (NSAttributedString *)attributedStringFrom:(id<WebDOMNode>)start startOffset:(int)startOffset to:(id<WebDOMNode>)end endOffset:(int)endOffset
 {
-    return KWQKHTMLPart::attributedString([(WebCoreDOMNode *)startNode impl], startOffset, [(WebCoreDOMNode *)endNode impl], endOffset);
+    WebCoreDOMNode *startNode = start;
+    WebCoreDOMNode *endNode = end;
+    return _part->attributedString([startNode impl], startOffset, [endNode impl], endOffset);
 }
 
 - (id<WebDOMNode>)selectionStart
@@ -829,6 +967,11 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
 }
 
 - (NSRect)selectionRect
+{
+    return _part->selectionRect(); 
+}
+
+- (NSRect)visibleSelectionRect
 {
     KHTMLView *view = _part->view();
     if (!view) {
@@ -848,7 +991,7 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
         return nil;
     }
 
-    NSRect rect = [self selectionRect];
+    NSRect rect = [self visibleSelectionRect];
     NSRect bounds = [view bounds];
     NSImage *selectionImage = [[[NSImage alloc] initWithSize:rect.size] autorelease];
     [selectionImage setFlipped:YES];
@@ -872,7 +1015,7 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
 
 - (void)setName:(NSString *)name
 {
-    _part->setName(QString::fromNSString(name));
+    _part->KHTMLPart::setName(QString::fromNSString(name));
 }
 
 - (NSString *)name
@@ -880,29 +1023,14 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
     return _part->name().getNSString();
 }
 
-- (NSString *)URL
+- (NSURL *)URL
 {
-    return _part->url().url().getNSString();
+    return _part->url().getNSURL();
 }
 
 - (NSString *)referrer
 {
-    // Do not allow file URLs to be used as referrers as that is potentially a security issue
-    NSString *referrer = _part->referrer().getNSString();
-    BOOL isFileURL = [referrer rangeOfString:@"file:" options:(NSCaseInsensitiveSearch | NSAnchoredSearch)].location != NSNotFound;
-    return isFileURL ? nil : referrer;
-}
-
-- (int)frameBorderStyle
-{
-    KHTMLView *view = _part->view();
-    if (view) {
-        if (view->frameStyle() & QFrame::Sunken)
-            return SunkenFrameBorder;
-        if (view->frameStyle() & QFrame::Plain)
-            return PlainFrameBorder;
-    }
-    return NoFrameBorder;
+    return _part->referrer().getNSString();
 }
 
 + (NSString *)stringWithData:(NSData *)data textEncoding:(CFStringEncoding)textEncoding
@@ -919,12 +1047,27 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
     return [WebCoreBridge stringWithData:data textEncoding:textEncoding];
 }
 
++ (void)updateAllViews
+{
+    for (QPtrListIterator<KWQKHTMLPart> it(KWQKHTMLPart::instances()); it.current(); ++it) {
+        KWQKHTMLPart *part = it.current();
+        [part->bridge() setNeedsReapplyStyles];
+    }
+}
+
 - (BOOL)needsLayout
 {
     RenderObject *renderer = _part->renderer();
     return renderer ? renderer->needsLayout() : false;
 }
 
+- (void)setNeedsLayout
+{
+    RenderObject *renderer = _part->renderer();
+    if (renderer)
+        renderer->setNeedsLayout(true);
+}
+
 - (BOOL)interceptKeyEvent:(NSEvent *)event toView:(NSView *)view
 {
     return _part->keyEvent(event);
@@ -945,6 +1088,11 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
     return _part->usesInactiveTextBackgroundColor();
 }
 
+- (void)setShowsFirstResponder:(BOOL)flag
+{
+    _part->setShowsFirstResponder(flag);
+}
+
 - (void)setShouldCreateRenderers:(BOOL)f
 {
     _shouldCreateRenderers = f;
@@ -963,6 +1111,26 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
         return KWQNumberOfPendingOrLoadingRequests (doc->docLoader());
     return 0;
 }
-    
+
+- (NSColor *)bodyBackgroundColor
+{
+    return _part->bodyBackgroundColor();
+}
+
+- (void)adjustViewSize
+{
+    KHTMLView *view = _part->view();
+    if (view)
+        view->adjustViewSize();
+}
+
+-(id)accessibilityTree
+{
+    if (!_part || !_part->xmlDocImpl()) return nil;
+    RenderCanvas* root = static_cast<khtml::RenderCanvas *>(_part->xmlDocImpl()->renderer());
+    if (!root) return nil;
+    return _part->xmlDocImpl()->getOrCreateAccObjectCache()->accObject(root);
+}
+
 
 @end
